# coding=utf8
"""
A mini-demo of what wsme can do.

To run it::

    python setup.py develop

Then::

    python demo.py
"""

from wsme import WSRoot, expose, validate
from wsme.types import File

import bottle

from six import u

import logging


class Person(object):
    id = int
    firstname = unicode
    lastname = unicode

    hobbies = [unicode]

    def __repr__(self):
        return "Person(%s, %s %s, %s)" % (
            self.id,
            self.firstname, self.lastname,
            self.hobbies
        )


class DemoRoot(WSRoot):
    @expose(int)
    @validate(int, int)
    def multiply(self, a, b):
        return a * b

    @expose(File)
    @validate(File)
    def echofile(self, afile):
        return afile

    @expose(unicode)
    def helloworld(self):
        return u"Здраво, свете (<- Hello World in Serbian !)"

    @expose(Person)
    def getperson(self):
        p = Person()
        p.id = 12
        p.firstname = u'Ross'
        p.lastname = u'Geler'
        p.hobbies = []
        print p
        return p

    @expose([Person])
    def listpersons(self):
        p = Person()
        p.id = 12
        p.firstname = u('Ross')
        p.lastname = u('Geler')
        r = [p]
        p = Person()
        p.id = 13
        p.firstname = u('Rachel')
        p.lastname = u('Green')
        r.append(p)
        print r
        return r

    @expose(Person)
    @validate(Person)
    def setperson(self, person):
        return person

    @expose([Person])
    @validate([Person])
    def setpersons(self, persons):
        print persons
        return persons


root = DemoRoot(webpath='/ws')

root.addprotocol('soap',
        tns='http://example.com/demo',
        typenamespace='http://example.com/demo/types',
        baseURL='http://127.0.0.1:8080/ws/',
)

root.addprotocol('restjson')

bottle.mount('/ws/', root.wsgiapp())

logging.basicConfig(level=logging.DEBUG)
bottle.run()



import cherrypy

from wsmeext.tg1 import wsexpose, wsvalidate
import wsmeext.tg1


__all__ = ['adapt', 'wsexpose', 'wsvalidate']


def scan_api(root=None):
    for baseurl, instance in cherrypy.tree.apps.items():
        path = [token for token in baseurl.split('/') if token]
        for i in wsmeext.tg1._scan_api(instance.root, path):
            yield i


def adapt(wsroot):
    wsroot._scan_api = scan_api
    controller = wsmeext.tg1.Controller(wsroot)
    return controller



# encoding=utf8

import unittest

from wsme import WSRoot
from wsme.protocol import getprotocol, CallContext, Protocol
import wsme.protocol


class DummyProtocol(Protocol):
    name = 'dummy'
    content_types = ['', None]

    def __init__(self):
        self.hits = 0

    def accept(self, req):
        return True

    def iter_calls(self, req):
        yield CallContext(req)

    def extract_path(self, context):
        return ['touch']

    def read_arguments(self, context):
        self.lastreq = context.request
        self.hits += 1
        return {}

    def encode_result(self, context, result):
        return str(result)

    def encode_error(self, context, infos):
        return str(infos)


def test_getprotocol():
    try:
        getprotocol('invalid')
        assert False, "ValueError was not raised"
    except ValueError:
        pass


class TestProtocols(unittest.TestCase):
    def test_register_protocol(self):
        wsme.protocol.register_protocol(DummyProtocol)
        assert wsme.protocol.registered_protocols['dummy'] == DummyProtocol

        r = WSRoot()
        assert len(r.protocols) == 0

        r.addprotocol('dummy')
        assert len(r.protocols) == 1
        assert r.protocols[0].__class__ == DummyProtocol

        r = WSRoot(['dummy'])
        assert len(r.protocols) == 1
        assert r.protocols[0].__class__ == DummyProtocol

    def test_Protocol(self):
        p = wsme.protocol.Protocol()
        assert p.iter_calls(None) is None
        assert p.extract_path(None) is None
        assert p.read_arguments(None) is None
        assert p.encode_result(None, None) is None
        assert p.encode_sample_value(None, None) == ('none', 'N/A')
        assert p.encode_sample_params(None) == ('none', 'N/A')
        assert p.encode_sample_result(None, None) == ('none', 'N/A')



import decimal
import datetime
import re
from six.moves import builtins, http_client

try:
    import dateutil.parser
except:
    dateutil = None  # noqa

date_re = r'(?P<year>-?\d{4,})-(?P<month>\d{2})-(?P<day>\d{2})'
time_re = r'(?P<hour>\d{2}):(?P<min>\d{2}):(?P<sec>\d{2})' + \
          r'(\.(?P<sec_frac>\d+))?'
tz_re = r'((?P<tz_sign>[+-])(?P<tz_hour>\d{2}):(?P<tz_min>\d{2}))' + \
        r'|(?P<tz_z>Z)'

datetime_re = re.compile(
    '%sT%s(%s)?' % (date_re, time_re, tz_re))
date_re = re.compile(date_re)
time_re = re.compile(time_re)


if hasattr(builtins, '_'):
    _ = builtins._
else:
    def _(s):
        return s


def parse_isodate(value):
    m = date_re.match(value)
    if m is None:
        raise ValueError("'%s' is not a legal date value" % (value))
    try:
        return datetime.date(
            int(m.group('year')),
            int(m.group('month')),
            int(m.group('day')))
    except ValueError:
        raise ValueError("'%s' is a out-of-range date" % (value))


def parse_isotime(value):
    m = time_re.match(value)
    if m is None:
        raise ValueError("'%s' is not a legal time value" % (value))
    try:
        ms = 0
        if m.group('sec_frac') is not None:
            f = decimal.Decimal('0.' + m.group('sec_frac'))
            f = str(f.quantize(decimal.Decimal('0.000001')))
            ms = int(f[2:])
        return datetime.time(
            int(m.group('hour')),
            int(m.group('min')),
            int(m.group('sec')),
            ms)
    except ValueError:
        raise ValueError("'%s' is a out-of-range time" % (value))


# TODO handle timezone
def parse_isodatetime(value):
    if dateutil:
        return dateutil.parser.parse(value)
    m = datetime_re.match(value)
    if m is None:
        raise ValueError("'%s' is not a legal datetime value" % (value))
    try:
        ms = 0
        if m.group('sec_frac') is not None:
            f = decimal.Decimal('0.' + m.group('sec_frac'))
            f = f.quantize(decimal.Decimal('0.000001'))
            ms = int(str(f)[2:])
        return datetime.datetime(
            int(m.group('year')),
            int(m.group('month')),
            int(m.group('day')),
            int(m.group('hour')),
            int(m.group('min')),
            int(m.group('sec')),
            ms)
    except ValueError:
        raise ValueError("'%s' is a out-of-range datetime" % (value))


def is_valid_code(code_value):
    """
    This function checks if incoming value in http response codes range.
    """
    return code_value in http_client.responses


def is_client_error(code):
    """ Checks client error code (RFC 2616)."""
    return 400 <= code < 500


try:
    from collections import OrderedDict
except ImportError:
    from ordereddict import OrderedDict  # noqa



import six

from wsme.utils import _


class ClientSideError(RuntimeError):
    def __init__(self, msg=None, status_code=400):
        self.msg = msg
        self.code = status_code
        super(ClientSideError, self).__init__(self.faultstring)

    @property
    def faultstring(self):
        if self.msg is None:
            return str(self)
        elif isinstance(self.msg, six.text_type):
            return self.msg
        else:
            return six.u(self.msg)


class InvalidInput(ClientSideError):
    def __init__(self, fieldname, value, msg=''):
        self.fieldname = fieldname
        self.value = value
        super(InvalidInput, self).__init__(msg)

    @property
    def faultstring(self):
        return _(six.u(
            "Invalid input for field/attribute %s. Value: '%s'. %s")
        ) % (self.fieldname, self.value, self.msg)


class MissingArgument(ClientSideError):
    def __init__(self, argname, msg=''):
        self.argname = argname
        super(MissingArgument, self).__init__(msg)

    @property
    def faultstring(self):
        return _(six.u('Missing argument: "%s"%s')) % (
            self.argname, self.msg and ": " + self.msg or "")


class UnknownArgument(ClientSideError):
    def __init__(self, argname, msg=''):
        self.argname = argname
        super(UnknownArgument, self).__init__(msg)

    @property
    def faultstring(self):
        return _(six.u('Unknown argument: "%s"%s')) % (
            self.argname, self.msg and ": " + self.msg or "")


class UnknownFunction(ClientSideError):
    def __init__(self, name):
        self.name = name
        super(UnknownFunction, self).__init__()

    @property
    def faultstring(self):
        return _(six.u("Unknown function name: %s")) % (self.name)



import mock
import unittest

from wsme import exc
from wsme.rest import args
from wsme.rest import json


class TestArgs(unittest.TestCase):

    def test_args_from_body(self):

        funcdef = mock.MagicMock()
        body = mock.MagicMock()
        mimetype = "application/json"
        funcdef.ignore_extra_args = True
        json.parse = mock.MagicMock()
        json.parse.side_effect = (exc.UnknownArgument(""))
        resp = args.args_from_body(funcdef, body, mimetype)
        self.assertEqual(resp, ((), {}))



from __future__ import absolute_import

import functools
import inspect
import sys

import wsme
import wsme.rest.args
import wsme.rest.json
import wsme.rest.xml

import pecan

from wsme.utils import is_valid_code


class JSonRenderer(object):
    @staticmethod
    def __init__(path, extra_vars):
        pass

    @staticmethod
    def render(template_path, namespace):
        if 'faultcode' in namespace:
            return wsme.rest.json.encode_error(None, namespace)
        return wsme.rest.json.encode_result(
            namespace['result'],
            namespace['datatype']
        )


class XMLRenderer(object):
    @staticmethod
    def __init__(path, extra_vars):
        pass

    @staticmethod
    def render(template_path, namespace):
        if 'faultcode' in namespace:
            return wsme.rest.xml.encode_error(None, namespace)
        return wsme.rest.xml.encode_result(
            namespace['result'],
            namespace['datatype']
        )

pecan.templating._builtin_renderers['wsmejson'] = JSonRenderer
pecan.templating._builtin_renderers['wsmexml'] = XMLRenderer

pecan_json_decorate = pecan.expose(
    template='wsmejson:',
    content_type='application/json',
    generic=False)
pecan_xml_decorate = pecan.expose(
    template='wsmexml:',
    content_type='application/xml',
    generic=False
)
pecan_text_xml_decorate = pecan.expose(
    template='wsmexml:',
    content_type='text/xml',
    generic=False
)


def wsexpose(*args, **kwargs):
    sig = wsme.signature(*args, **kwargs)

    def decorate(f):
        sig(f)
        funcdef = wsme.api.FunctionDefinition.get(f)
        funcdef.resolve_types(wsme.types.registry)

        @functools.wraps(f)
        def callfunction(self, *args, **kwargs):
            try:
                args, kwargs = wsme.rest.args.get_args(
                    funcdef, args, kwargs, pecan.request.params, None,
                    pecan.request.body, pecan.request.content_type
                )
                if funcdef.pass_request:
                    kwargs[funcdef.pass_request] = pecan.request
                result = f(self, *args, **kwargs)

                # NOTE: Support setting of status_code with default 201
                pecan.response.status = funcdef.status_code
                if isinstance(result, wsme.api.Response):
                    pecan.response.status = result.status_code
                    result = result.obj

            except:
                try:
                    exception_info = sys.exc_info()
                    orig_exception = exception_info[1]
                    orig_code = getattr(orig_exception, 'code', None)
                    data = wsme.api.format_exception(
                        exception_info,
                        pecan.conf.get('wsme', {}).get('debug', False)
                    )
                finally:
                    del exception_info

                if orig_code and is_valid_code(orig_code):
                    pecan.response.status = orig_code
                else:
                    pecan.response.status = 500

                return data

            if funcdef.return_type is None:
                pecan.request.pecan['content_type'] = None
                pecan.response.content_type = None
                return ''

            return dict(
                datatype=funcdef.return_type,
                result=result
            )

        pecan_xml_decorate(callfunction)
        pecan_text_xml_decorate(callfunction)
        pecan_json_decorate(callfunction)
        pecan.util._cfg(callfunction)['argspec'] = inspect.getargspec(f)
        callfunction._wsme_definition = funcdef
        return callfunction

    return decorate



import base64
import datetime
import decimal

import wsme.tests.protocol

try:
    import simplejson as json
except:
    import json  # noqa

from wsme.rest.json import fromjson, tojson
from wsme.utils import parse_isodatetime, parse_isotime, parse_isodate
from wsme.types import isarray, isdict, isusertype, register_type
from wsme.rest import expose, validate


import six
from six import b, u

if six.PY3:
    from urllib.parse import urlencode
else:
    from urllib import urlencode  # noqa


def prepare_value(value, datatype):
    if isinstance(datatype, list):
        return [prepare_value(item, datatype[0]) for item in value]
    if isinstance(datatype, dict):
        key_type, value_type = list(datatype.items())[0]
        return dict((
            (prepare_value(item[0], key_type),
                prepare_value(item[1], value_type))
            for item in value.items()
        ))
    if datatype in (datetime.date, datetime.time, datetime.datetime):
        return value.isoformat()
    if datatype == decimal.Decimal:
        return str(value)
    if datatype == wsme.types.binary:
        return base64.encodestring(value).decode('ascii')
    if datatype == wsme.types.bytes:
        return value.decode('ascii')
    return value


def prepare_result(value, datatype):
    print(value, datatype)
    if value is None:
        return None
    if datatype == wsme.types.binary:
        return base64.decodestring(value.encode('ascii'))
    if isusertype(datatype):
        datatype = datatype.basetype
    if isinstance(datatype, list):
        return [prepare_result(item, datatype[0]) for item in value]
    if isarray(datatype):
        return [prepare_result(item, datatype.item_type) for item in value]
    if isinstance(datatype, dict):
        return dict((
            (prepare_result(item[0], list(datatype.keys())[0]),
                prepare_result(item[1], list(datatype.values())[0]))
            for item in value.items()
        ))
    if isdict(datatype):
        return dict((
            (prepare_result(item[0], datatype.key_type),
                prepare_result(item[1], datatype.value_type))
            for item in value.items()
        ))
    if datatype == datetime.date:
        return parse_isodate(value)
    if datatype == datetime.time:
        return parse_isotime(value)
    if datatype == datetime.datetime:
        return parse_isodatetime(value)
    if hasattr(datatype, '_wsme_attributes'):
        for attr in datatype._wsme_attributes:
            if attr.key not in value:
                continue
            value[attr.key] = prepare_result(value[attr.key], attr.datatype)
        return value
    if datatype == wsme.types.bytes:
        return value.encode('ascii')
    if type(value) != datatype:
        print(type(value), datatype)
        return datatype(value)
    return value


class Obj(wsme.types.Base):
    id = int
    name = wsme.types.text


class CRUDResult(object):
    data = Obj
    message = wsme.types.text

    def __init__(self, data=wsme.types.Unset, message=wsme.types.Unset):
        self.data = data
        self.message = message


class MiniCrud(object):
    @expose(CRUDResult, method='PUT')
    @validate(Obj)
    def create(self, data):
        print(repr(data))
        return CRUDResult(data, u('create'))

    @expose(CRUDResult, method='GET', ignore_extra_args=True)
    @validate(Obj)
    def read(self, ref):
        print(repr(ref))
        if ref.id == 1:
            ref.name = u('test')
        return CRUDResult(ref, u('read'))

    @expose(CRUDResult, method='POST')
    @validate(Obj)
    def update(self, data):
        print(repr(data))
        return CRUDResult(data, u('update'))

    @expose(CRUDResult, wsme.types.text, body=Obj)
    def update_with_body(self, msg, data):
        print(repr(data))
        return CRUDResult(data, msg)

    @expose(CRUDResult, method='DELETE')
    @validate(Obj)
    def delete(self, ref):
        print(repr(ref))
        if ref.id == 1:
            ref.name = u('test')
        return CRUDResult(ref, u('delete'))

wsme.tests.protocol.WSTestRoot.crud = MiniCrud()


class TestRestJson(wsme.tests.protocol.RestOnlyProtocolTestCase):
    protocol = 'restjson'

    def call(self, fpath, _rt=None, _accept=None, _no_result_decode=False,
             body=None, **kw):
        if body:
            if isinstance(body, tuple):
                body, datatype = body
            else:
                datatype = type(body)
            body = prepare_value(body, datatype)
            content = json.dumps(body)
        else:
            for key in kw:
                if isinstance(kw[key], tuple):
                    value, datatype = kw[key]
                else:
                    value = kw[key]
                    datatype = type(value)
                kw[key] = prepare_value(value, datatype)
            content = json.dumps(kw)
        headers = {
            'Content-Type': 'application/json',
        }
        if _accept is not None:
            headers["Accept"] = _accept
        res = self.app.post(
            '/' + fpath,
            content,
            headers=headers,
            expect_errors=True)
        print("Received:", res.body)

        if _no_result_decode:
            return res

        r = json.loads(res.text)
        if res.status_int == 200:
            if _rt and r:
                r = prepare_result(r, _rt)
            return r
        else:
            raise wsme.tests.protocol.CallException(
                r['faultcode'],
                r['faultstring'],
                r.get('debuginfo')
            )

        return json.loads(res.text)

    def test_fromjson(self):
        assert fromjson(str, None) is None

    def test_keyargs(self):
        r = self.app.get('/argtypes/setint.json?value=2')
        print(r)
        assert json.loads(r.text) == 2

        nestedarray = 'value[0].inner.aint=54&value[1].inner.aint=55'
        r = self.app.get('/argtypes/setnestedarray.json?' + nestedarray)
        print(r)
        assert json.loads(r.text) == [
            {'inner': {'aint': 54}},
            {'inner': {'aint': 55}}]

    def test_form_urlencoded_args(self):
        params = {
            'value[0].inner.aint': 54,
            'value[1].inner.aint': 55
        }
        body = urlencode(params)
        r = self.app.post(
            '/argtypes/setnestedarray.json',
            body,
            headers={'Content-Type': 'application/x-www-form-urlencoded'}
        )
        print(r)

        assert json.loads(r.text) == [
            {'inner': {'aint': 54}},
            {'inner': {'aint': 55}}]

    def test_body_and_params(self):
        r = self.app.post('/argtypes/setint.json?value=2', '{"value": 2}',
                          headers={"Content-Type": "application/json"},
                          expect_errors=True)
        print(r)
        assert r.status_int == 400
        assert json.loads(r.text)['faultstring'] == \
            "Parameter value was given several times"

    def test_inline_body(self):
        params = urlencode({'__body__': '{"value": 4}'})
        r = self.app.get('/argtypes/setint.json?' + params)
        print(r)
        assert json.loads(r.text) == 4

    def test_empty_body(self):
        params = urlencode({'__body__': ''})
        r = self.app.get('/returntypes/getint.json?' + params)
        print(r)
        assert json.loads(r.text) == 2

    def test_unknown_arg(self):
        r = self.app.post('/returntypes/getint.json', '{"a": 2}',
                          headers={"Content-Type": "application/json"},
                          expect_errors=True)
        print(r)
        assert r.status_int == 400
        assert json.loads(r.text)['faultstring'].startswith(
            "Unknown argument:"
        )

        r = self.app.get('/returntypes/getint.json?a=2', expect_errors=True)
        print(r)
        assert r.status_int == 400
        assert json.loads(r.text)['faultstring'].startswith(
            "Unknown argument:"
        )

    def test_unset_attrs(self):
        class AType(object):
            attr = int

        wsme.types.register_type(AType)

        j = tojson(AType, AType())
        assert j == {}

    def test_array_tojson(self):
        assert tojson([int], None) is None
        assert tojson([int], []) == []
        assert tojson([str], ['1', '4']) == ['1', '4']

    def test_dict_tojson(self):
        assert tojson({int: str}, None) is None
        assert tojson({int: str}, {5: '5'}) == {5: '5'}

    def test_None_tojson(self):
        for dt in (datetime.date, datetime.time, datetime.datetime,
                   decimal.Decimal):
            assert tojson(dt, None) is None

    def test_None_fromjson(self):
        for dt in (str, int, datetime.date, datetime.time, datetime.datetime,
                   decimal.Decimal, [int], {int: int}):
            assert fromjson(dt, None) is None

    def test_nest_result(self):
        self.root.protocols[0].nest_result = True
        r = self.app.get('/returntypes/getint.json')
        print(r)
        assert json.loads(r.text) == {"result": 2}

    def test_encode_sample_value(self):
        class MyType(object):
            aint = int
            astr = str

        register_type(MyType)

        v = MyType()
        v.aint = 4
        v.astr = 's'

        r = wsme.rest.json.encode_sample_value(MyType, v, True)
        print(r)
        assert r[0] == ('javascript')
        assert r[1] == json.dumps({'aint': 4, 'astr': 's'}, ensure_ascii=False,
                                  indent=4, sort_keys=True)

    def test_bytes_tojson(self):
        assert tojson(wsme.types.bytes, None) is None
        assert tojson(wsme.types.bytes, b('ascii')) == u('ascii')

    def test_encode_sample_params(self):
        r = wsme.rest.json.encode_sample_params(
            [('a', int, 2)], True
        )
        assert r[0] == 'javascript', r[0]
        assert r[1] == '''{
    "a": 2
}''', r[1]

    def test_encode_sample_result(self):
        r = wsme.rest.json.encode_sample_result(
            int, 2, True
        )
        assert r[0] == 'javascript', r[0]
        assert r[1] == '''2'''

    def test_PUT(self):
        data = {"id": 1, "name": u("test")}
        content = json.dumps(dict(data=data))
        headers = {
            'Content-Type': 'application/json',
        }
        res = self.app.put(
            '/crud',
            content,
            headers=headers,
            expect_errors=False)
        print("Received:", res.body)
        result = json.loads(res.text)
        print(result)
        assert result['data']['id'] == 1
        assert result['data']['name'] == u("test")
        assert result['message'] == "create"

    def test_GET(self):
        headers = {
            'Content-Type': 'application/json',
        }
        res = self.app.get(
            '/crud?ref.id=1',
            headers=headers,
            expect_errors=False)
        print("Received:", res.body)
        result = json.loads(res.text)
        print(result)
        assert result['data']['id'] == 1
        assert result['data']['name'] == u("test")
        assert result['message'] == "read"

    def test_POST(self):
        headers = {
            'Content-Type': 'application/json',
        }
        res = self.app.post(
            '/crud',
            json.dumps(dict(data=dict(id=1, name=u('test')))),
            headers=headers,
            expect_errors=False)
        print("Received:", res.body)
        result = json.loads(res.text)
        print(result)
        assert result['data']['id'] == 1
        assert result['data']['name'] == u("test")
        assert result['message'] == "update"

    def test_DELETE(self):
        res = self.app.delete(
            '/crud.json?ref.id=1',
            expect_errors=False)
        print("Received:", res.body)
        result = json.loads(res.text)
        print(result)
        assert result['data']['id'] == 1
        assert result['data']['name'] == u("test")
        assert result['message'] == "delete"

    def test_extra_arguments(self):
        headers = {
            'Content-Type': 'application/json',
        }
        res = self.app.get(
            '/crud?ref.id=1&extraarg=foo',
            headers=headers,
            expect_errors=False)
        print("Received:", res.body)
        result = json.loads(res.text)
        print(result)
        assert result['data']['id'] == 1
        assert result['data']['name'] == u("test")
        assert result['message'] == "read"

    def test_body_arg(self):
        headers = {
            'Content-Type': 'application/json',
        }
        res = self.app.post(
            '/crud/update_with_body?msg=hello',
            json.dumps(dict(id=1, name=u('test'))),
            headers=headers,
            expect_errors=False)
        print("Received:", res.body)
        result = json.loads(res.text)
        print(result)
        assert result['data']['id'] == 1
        assert result['data']['name'] == u("test")
        assert result['message'] == "hello"



import spyre
import spyre.middleware


class CTypeHeader(spyre.middleware.Middleware):
    def __call__(self, env):
        env.setdefault('spore.headers', [])
        env['spore.headers'].extend([
            ('Accept', 'application/json'),
            ('Content-Type', 'application/json')
        ])


demo = spyre.new_from_url('http://127.0.0.1:8080/ws/api.spore')
demo.enable(CTypeHeader)
demo.enable('format.Json')

print demo.helloworld().content



import datetime
import unittest

from wsme import utils


class TestUtils(unittest.TestCase):
    def test_parse_isodate(self):
        good_dates = [
            ('2008-02-01', datetime.date(2008, 2, 1)),
            ('2009-01-04', datetime.date(2009, 1, 4)),
        ]
        ill_formatted_dates = [
            '24-12-2004'
        ]
        out_of_range_dates = [
            '0000-00-00',
            '2012-02-30',
        ]
        for s, d in good_dates:
            assert utils.parse_isodate(s) == d
        for s in ill_formatted_dates + out_of_range_dates:
            self.assertRaises(ValueError, utils.parse_isodate, s)

    def test_parse_isotime(self):
        good_times = [
            ('12:03:54', datetime.time(12, 3, 54)),
            ('23:59:59.000004', datetime.time(23, 59, 59, 4)),
        ]
        ill_formatted_times = [
            '24-12-2004'
        ]
        out_of_range_times = [
            '32:12:00',
            '00:54:60',
        ]
        for s, t in good_times:
            assert utils.parse_isotime(s) == t
        for s in ill_formatted_times + out_of_range_times:
            self.assertRaises(ValueError, utils.parse_isotime, s)

    def test_parse_isodatetime(self):
        good_datetimes = [
            ('2008-02-12T12:03:54',
                datetime.datetime(2008, 2, 12, 12, 3, 54)),
            ('2012-05-14T23:59:59.000004',
                datetime.datetime(2012, 5, 14, 23, 59, 59, 4)),
        ]
        ill_formatted_datetimes = [
            '24-12-2004'
        ]
        out_of_range_datetimes = [
            '2008-02-12T32:12:00',
            '2012-13-12T00:54:60',
        ]
        for s, t in good_datetimes:
            assert utils.parse_isodatetime(s) == t
        for s in ill_formatted_datetimes + out_of_range_datetimes:
            self.assertRaises(ValueError, utils.parse_isodatetime, s)

    def test_validator_with_valid_code(self):
        valid_code = 404
        self.assertTrue(
            utils.is_valid_code(valid_code),
            "Valid status code not detected"
        )

    def test_validator_with_invalid_int_code(self):
        invalid_int_code = 648
        self.assertFalse(
            utils.is_valid_code(invalid_int_code),
            "Invalid status code not detected"
        )

    def test_validator_with_invalid_str_code(self):
        invalid_str_code = '404'
        self.assertFalse(
            utils.is_valid_code(invalid_str_code),
            "Invalid status code not detected"
        )



import datetime

try:
    import json
except ImportError:
    import simplejson as json

from webtest import TestApp

from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Column, Integer, Unicode, Date, ForeignKey
from sqlalchemy.orm import relation

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker, scoped_session

from wsme import WSRoot
import wsme.types

from wsmeext.sqlalchemy.types import generate_types
from wsmeext.sqlalchemy.controllers import CRUDController

from six import u

engine = create_engine('sqlite:///')
DBSession = scoped_session(sessionmaker(autocommit=False, autoflush=False,
                                        bind=engine))
DBBase = declarative_base()

registry = wsme.types.Registry()


class DBPerson(DBBase):
    __tablename__ = 'person'

    id = Column(Integer, primary_key=True)
    name = Column(Unicode(50))
    birthdate = Column(Date)

    addresses = relation('DBAddress')


class DBAddress(DBBase):
    __tablename__ = 'address'

    id = Column(Integer, primary_key=True)

    _person_id = Column('person_id', ForeignKey(DBPerson.id))

    street = Column(Unicode(50))
    city = Column(Unicode(50))

    person = relation(DBPerson)


globals().update(
    generate_types(DBPerson, DBAddress, makename=lambda s: s[2:],
                   registry=registry))


class PersonController(CRUDController):
    __saclass__ = DBPerson
    __dbsession__ = DBSession
    __registry__ = registry


class AddressController(CRUDController):
    __saclass__ = DBAddress
    __dbsession__ = DBSession
    __registry__ = registry


class Root(WSRoot):
    __registry__ = registry

    person = PersonController()
    address = AddressController()


class TestCRUDController():
    def setUp(self):
        DBBase.metadata.create_all(DBSession.bind)

        self.root = Root()
        self.root.getapi()
        self.root.addprotocol('restjson')

        self.app = TestApp(self.root.wsgiapp())

    def tearDown(self):
        DBBase.metadata.drop_all(DBSession.bind)

    def test_create(self):
        data = dict(data=dict(
            name=u('Pierre-Joseph'),
            birthdate=u('1809-01-15')
        ))
        r = self.app.post('/person/create', json.dumps(data),
                          headers={'Content-Type': 'application/json'})
        r = json.loads(r.text)
        print(r)
        assert r['name'] == u('Pierre-Joseph')
        assert r['birthdate'] == u('1809-01-15')

    def test_PUT(self):
        data = dict(data=dict(
            name=u('Pierre-Joseph'),
            birthdate=u('1809-01-15')
        ))
        r = self.app.put('/person', json.dumps(data),
                         headers={'Content-Type': 'application/json'})
        r = json.loads(r.text)
        print(r)
        assert r['name'] == u('Pierre-Joseph')
        assert r['birthdate'] == u('1809-01-15')

    def test_read(self):
        p = DBPerson(
            name=u('Pierre-Joseph'),
            birthdate=datetime.date(1809, 1, 15))
        DBSession.add(p)
        DBSession.flush()
        pid = p.id
        r = self.app.post('/person/read', '{"ref": {"id": %s}}' % pid,
                          headers={'Content-Type': 'application/json'})
        r = json.loads(r.text)
        print(r)
        assert r['name'] == u('Pierre-Joseph')
        assert r['birthdate'] == u('1809-01-15')

    def test_GET(self):
        p = DBPerson(
            name=u('Pierre-Joseph'),
            birthdate=datetime.date(1809, 1, 15))
        DBSession.add(p)
        DBSession.flush()
        pid = p.id
        r = self.app.get('/person?ref.id=%s' % pid,
                         headers={'Content-Type': 'application/json'})
        r = json.loads(r.text)
        print(r)
        assert r['name'] == u('Pierre-Joseph')
        assert r['birthdate'] == u('1809-01-15')

    def test_update(self):
        p = DBPerson(
            name=u('Pierre-Joseph'),
            birthdate=datetime.date(1809, 1, 15))
        DBSession.add(p)
        DBSession.flush()
        pid = p.id
        data = {
            "id": pid,
            "name": u('Pierre-Joseph Proudon')
        }
        r = self.app.post('/person/update', json.dumps(dict(data=data)),
                          headers={'Content-Type': 'application/json'})
        r = json.loads(r.text)
        print(r)
        assert r['name'] == u('Pierre-Joseph Proudon')
        assert r['birthdate'] == u('1809-01-15')

    def test_POST(self):
        p = DBPerson(
            name=u('Pierre-Joseph'),
            birthdate=datetime.date(1809, 1, 15))
        DBSession.add(p)
        DBSession.flush()
        pid = p.id
        data = {
            "id": pid,
            "name": u('Pierre-Joseph Proudon')
        }
        r = self.app.post('/person', json.dumps(dict(data=data)),
                          headers={'Content-Type': 'application/json'})
        r = json.loads(r.text)
        print(r)
        assert r['name'] == u('Pierre-Joseph Proudon')
        assert r['birthdate'] == u('1809-01-15')

    def test_delete(self):
        p = DBPerson(
            name=u('Pierre-Joseph'),
            birthdate=datetime.date(1809, 1, 15))
        DBSession.add(p)
        DBSession.flush()
        pid = p.id
        r = self.app.post('/person/delete', json.dumps(
            dict(ref=dict(id=pid))),
            headers={
                'Content-Type': 'application/json'
            })
        print(r)
        assert DBSession.query(DBPerson).get(pid) is None

    def test_DELETE(self):
        p = DBPerson(
            name=u('Pierre-Joseph'),
            birthdate=datetime.date(1809, 1, 15))
        DBSession.add(p)
        DBSession.flush()
        pid = p.id
        r = self.app.delete('/person?ref.id=%s' % pid,
                            headers={'Content-Type': 'application/json'})
        print(r)
        assert DBSession.query(DBPerson).get(pid) is None

    def test_nothing(self):
        pass



from wsme.exc import MissingArgument


def check_arguments(funcdef, args, kw):
    """Check if some arguments are missing"""
    assert len(args) == 0
    for arg in funcdef.arguments:
        if arg.mandatory and arg.name not in kw:
            raise MissingArgument(arg.name)



import datetime
import decimal
import logging

import six

from sqlalchemy.orm import class_mapper
from sqlalchemy.orm.properties import ColumnProperty, RelationProperty

import sqlalchemy.types

import wsme.types

log = logging.getLogger(__name__)


class SQLAlchemyRegistry(object):
    @classmethod
    def get(cls, registry):
        if not hasattr(registry, 'sqlalchemy'):
            registry.sqlalchemy = cls()
        return registry.sqlalchemy

    def __init__(self):
        self.types = {}
        self.satypeclasses = {
            sqlalchemy.types.Integer: int,
            sqlalchemy.types.Boolean: bool,
            sqlalchemy.types.Float: float,
            sqlalchemy.types.Numeric: decimal.Decimal,
            sqlalchemy.types.Date: datetime.date,
            sqlalchemy.types.Time: datetime.time,
            sqlalchemy.types.DateTime: datetime.datetime,
            sqlalchemy.types.String: wsme.types.text,
            sqlalchemy.types.Unicode: wsme.types.text,
        }

    def getdatatype(self, sadatatype):
        if sadatatype.__class__ in self.satypeclasses:
            return self.satypeclasses[sadatatype.__class__]
        elif sadatatype in self.types:
            return self.types[sadatatype]
        else:
            return sadatatype.__name__


def register_saclass(registry, saclass, typename=None):
    """Associate a webservice type name to a SQLAlchemy mapped class.
    The default typename if the saclass name itself.
    """
    if typename is None:
        typename = saclass.__name__

    SQLAlchemyRegistry.get(registry).types[saclass] = typename


class wsattr(wsme.types.wsattr):
    def __init__(self, datatype, saproperty=None, **kw):
        super(wsattr, self).__init__(datatype, **kw)
        self.saname = saproperty.key
        self.saproperty = saproperty
        self.isrelation = isinstance(saproperty, RelationProperty)


def make_wsattr(registry, saproperty):
    datatype = None
    if isinstance(saproperty, ColumnProperty):
        if len(saproperty.columns) > 1:
            log.warning("Cannot handle multi-column ColumnProperty")
            return None
        datatype = SQLAlchemyRegistry.get(registry).getdatatype(
            saproperty.columns[0].type)
    elif isinstance(saproperty, RelationProperty):
        other_saclass = saproperty.mapper.class_
        datatype = SQLAlchemyRegistry.get(registry).getdatatype(other_saclass)
        if saproperty.uselist:
            datatype = [datatype]
    else:
        log.warning("Don't know how to handle %s attributes" %
                    saproperty.__class__)

    if datatype:
        return wsattr(datatype, saproperty)


class BaseMeta(wsme.types.BaseMeta):
    def __new__(cls, name, bases, dct):
        if '__registry__' not in dct:
            dct['__registry__'] = wsme.types.registry
        return type.__new__(cls, name, bases, dct)

    def __init__(cls, name, bases, dct):
        saclass = getattr(cls, '__saclass__', None)
        if saclass:
            mapper = class_mapper(saclass)
            cls._pkey_attrs = []
            cls._ref_attrs = []
            for prop in mapper.iterate_properties:
                key = prop.key
                if hasattr(cls, key):
                    continue
                if key.startswith('_'):
                    continue
                attr = make_wsattr(cls.__registry__, prop)
                if attr is not None:
                    setattr(cls, key, attr)

                if attr and isinstance(prop, ColumnProperty) and \
                        prop.columns[0] in mapper.primary_key:
                    cls._pkey_attrs.append(attr)
                    cls._ref_attrs.append(attr)

            register_saclass(cls.__registry__, cls.__saclass__, cls.__name__)
        super(BaseMeta, cls).__init__(name, bases, dct)


class Base(six.with_metaclass(BaseMeta, wsme.types.Base)):
    def __init__(self, instance=None, keyonly=False, attrs=None, eagerload=[]):
        if instance:
            self.from_instance(instance, keyonly, attrs, eagerload)

    def from_instance(self, instance, keyonly=False, attrs=None, eagerload=[]):
        if keyonly:
            attrs = self._pkey_attrs + self._ref_attrs
        for attr in self._wsme_attributes:
            if not isinstance(attr, wsattr):
                continue
            if attrs and not attr.isrelation and attr.name not in attrs:
                continue
            if attr.isrelation and attr.name not in eagerload:
                continue
            value = getattr(instance, attr.saname)
            if attr.isrelation:
                attr_keyonly = attr.name not in eagerload
                attr_attrs = None
                attr_eagerload = []
                if not attr_keyonly:
                    attr_attrs = [
                        aname[len(attr.name) + 1:]
                        for aname in attrs
                        if aname.startswith(attr.name + '.')
                    ]
                    attr_eagerload = [
                        aname[len(attr.name) + 1:]
                        for aname in eagerload
                        if aname.startswith(attr.name + '.')
                    ]
                if attr.saproperty.uselist:
                    value = [
                        attr.datatype.item_type(
                            o,
                            keyonly=attr_keyonly,
                            attrs=attr_attrs,
                            eagerload=attr_eagerload
                        )
                        for o in value
                    ]
                else:
                    value = attr.datatype(
                        value,
                        keyonly=attr_keyonly,
                        attrs=attr_attrs,
                        eagerload=attr_eagerload
                    )
            attr.__set__(self, value)

    def to_instance(self, instance):
        for attr in self._wsme_attributes:
            if isinstance(attr, wsattr):
                value = attr.__get__(self, self.__class__)
                if value is not wsme.types.Unset:
                    setattr(instance, attr.saname, value)

    def get_ref_criterion(self):
        """Returns a criterion that match a database object
        having the pkey/ref attribute values of this webservice object"""
        criterions = []
        for attr in self._pkey_attrs + self._ref_attrs:
            value = attr.__get__(self, self.__class__)
            if value is not wsme.types.Unset:
                criterions.append(attr.saproperty == value)


def generate_types(*classes, **kw):
    registry = kw.pop('registry', wsme.types.registry)
    prefix = kw.pop('prefix', '')
    postfix = kw.pop('postfix', '')
    makename = kw.pop('makename', lambda s: prefix + s + postfix)

    newtypes = {}
    for c in classes:
        if isinstance(c, list):
            newtypes.update(generate_types(c))
        else:
            name = makename(c.__name__)
            newtypes[name] = BaseMeta(name, (Base, ), {
                '__saclass__': c,
                '__registry__': registry
            })
    return newtypes



from wsme.api import signature
from wsme.rest import expose, validate
from wsme.root import WSRoot
from wsme.types import wsattr, wsproperty, Unset

__all__ = [
    'expose', 'validate', 'signature',
    'WSRoot',
    'wsattr', 'wsproperty', 'Unset'
]



import os.path
import logging

from wsme.utils import OrderedDict
from wsme.protocol import CallContext, Protocol

import wsme.rest
import wsme.rest.args
import wsme.runtime

log = logging.getLogger(__name__)


class RestProtocol(Protocol):
    name = 'rest'
    displayname = 'REST'
    dataformats = ['json', 'xml']
    content_types = ['application/json', 'text/xml']

    def __init__(self, dataformats=None):
        if dataformats is None:
            dataformats = RestProtocol.dataformats

        self.dataformats = OrderedDict()
        self.content_types = []

        for dataformat in dataformats:
            __import__('wsme.rest.' + dataformat)
            dfmod = getattr(wsme.rest, dataformat)
            self.dataformats[dataformat] = dfmod
            self.content_types.extend(dfmod.accept_content_types)

    def accept(self, request):
        for dataformat in self.dataformats:
            if request.path.endswith('.' + dataformat):
                return True
        if request.headers.get('Accept') in self.content_types:
            return True
        for ct in self.content_types:
            if request.headers['Content-Type'].startswith(ct):
                return True
        return False

    def iter_calls(self, request):
        context = CallContext(request)
        context.outformat = None
        ext = os.path.splitext(request.path.split('/')[-1])[1]
        inmime = request.content_type
        outmime = request.accept.best_match(self.content_types)

        outformat = None
        informat = None
        for dfname, df in self.dataformats.items():
            if ext == '.' + dfname:
                outformat = df
                if not inmime:
                    informat = df

        if outformat is None and request.accept:
            for dfname, df in self.dataformats.items():
                if outmime in df.accept_content_types:
                    outformat = df
                    if not inmime:
                        informat = df

        if outformat is None:
            for dfname, df in self.dataformats.items():
                if inmime == df.content_type:
                    outformat = df

        context.outformat = outformat
        context.outformat_options = {
            'nest_result': getattr(self, 'nest_result', False)
        }
        if not inmime and informat:
            inmime = informat.content_type
            log.debug("Inferred input type: %s" % inmime)
        context.inmime = inmime
        yield context

    def extract_path(self, context):
        path = context.request.path
        assert path.startswith(self.root._webpath)
        path = path[len(self.root._webpath):]
        path = path.strip('/').split('/')

        for dataformat in self.dataformats:
            if path[-1].endswith('.' + dataformat):
                path[-1] = path[-1][:-len(dataformat) - 1]

        # Check if the path is actually a function, and if not
        # see if the http method make a difference
        # TODO Re-think the function lookup phases. Here we are
        # doing the job that will be done in a later phase, which
        # is sub-optimal
        for p, fdef in self.root.getapi():
            if p == path:
                return path

        # No function at this path. Now check for function that have
        # this path as a prefix, and declared an http method
        for p, fdef in self.root.getapi():
            if len(p) == len(path) + 1 and p[:len(path)] == path and \
                    fdef.extra_options.get('method') == context.request.method:
                return p

        return path

    def read_arguments(self, context):
        request = context.request
        funcdef = context.funcdef

        body = None
        if request.content_length not in (None, 0, '0'):
            body = request.body
        if not body and '__body__' in request.params:
            body = request.params['__body__']

        args, kwargs = wsme.rest.args.combine_args(
            funcdef,
            (wsme.rest.args.args_from_params(funcdef, request.params),
             wsme.rest.args.args_from_body(funcdef, body, context.inmime))
        )
        wsme.runtime.check_arguments(funcdef, args, kwargs)
        return kwargs

    def encode_result(self, context, result):
        out = context.outformat.encode_result(
            result, context.funcdef.return_type,
            **context.outformat_options
        )
        return out

    def encode_error(self, context, errordetail):
        out = context.outformat.encode_error(
            context, errordetail
        )
        return out



"""
REST+Json protocol implementation.
"""
from __future__ import absolute_import
import datetime
import decimal

import six

from simplegeneric import generic

from wsme.types import Unset
import wsme.types
import wsme.utils
from wsme.exc import UnknownArgument, InvalidInput


try:
    import simplejson as json
except ImportError:
    import json  # noqa


content_type = 'application/json'
accept_content_types = [
    content_type,
    'text/javascript',
    'application/javascript'
]


@generic
def tojson(datatype, value):
    """
    A generic converter from python to jsonify-able datatypes.

    If a non-complex user specific type is to be used in the api,
    a specific tojson should be added::

        from wsme.protocol.restjson import tojson

        myspecialtype = object()

        @tojson.when_object(myspecialtype)
        def myspecialtype_tojson(datatype, value):
            return str(value)
    """
    if value is None:
        return None
    if wsme.types.iscomplex(datatype):
        d = dict()
        for attr in wsme.types.list_attributes(datatype):
            attr_value = getattr(value, attr.key)
            if attr_value is not Unset:
                d[attr.name] = tojson(attr.datatype, attr_value)
        return d
    elif wsme.types.isusertype(datatype):
        return tojson(datatype.basetype, datatype.tobasetype(value))
    return value


@tojson.when_object(wsme.types.bytes)
def bytes_tojson(datatype, value):
    if value is None:
        return None
    return value.decode('ascii')


@tojson.when_type(wsme.types.ArrayType)
def array_tojson(datatype, value):
    if value is None:
        return None
    return [tojson(datatype.item_type, item) for item in value]


@tojson.when_type(wsme.types.DictType)
def dict_tojson(datatype, value):
    if value is None:
        return None
    return dict((
        (tojson(datatype.key_type, item[0]),
            tojson(datatype.value_type, item[1]))
        for item in value.items()
    ))


@tojson.when_object(decimal.Decimal)
def decimal_tojson(datatype, value):
    if value is None:
        return None
    return str(value)


@tojson.when_object(datetime.date)
def date_tojson(datatype, value):
    if value is None:
        return None
    return value.isoformat()


@tojson.when_object(datetime.time)
def time_tojson(datatype, value):
    if value is None:
        return None
    return value.isoformat()


@tojson.when_object(datetime.datetime)
def datetime_tojson(datatype, value):
    if value is None:
        return None
    return value.isoformat()


@generic
def fromjson(datatype, value):
    """
    A generic converter from json base types to python datatype.

    If a non-complex user specific type is to be used in the api,
    a specific fromjson should be added::

        from wsme.protocol.restjson import fromjson

        class MySpecialType(object):
            pass

        @fromjson.when_object(MySpecialType)
        def myspecialtype_fromjson(datatype, value):
            return MySpecialType(value)
    """
    if value is None:
        return None
    if wsme.types.iscomplex(datatype):
        obj = datatype()
        for attrdef in wsme.types.list_attributes(datatype):
            if attrdef.name in value:
                val_fromjson = fromjson(attrdef.datatype, value[attrdef.name])
                if getattr(attrdef, 'readonly', False):
                    raise InvalidInput(attrdef.name, val_fromjson,
                                       "Cannot set read only field.")
                setattr(obj, attrdef.key, val_fromjson)
            elif attrdef.mandatory:
                raise InvalidInput(attrdef.name, None,
                                   "Mandatory field missing.")
        return wsme.types.validate_value(datatype, obj)
    elif wsme.types.isusertype(datatype):
        value = datatype.frombasetype(
            fromjson(datatype.basetype, value))
    return value


@fromjson.when_type(wsme.types.ArrayType)
def array_fromjson(datatype, value):
    if value is None:
        return None
    return [fromjson(datatype.item_type, item) for item in value]


@fromjson.when_type(wsme.types.DictType)
def dict_fromjson(datatype, value):
    if value is None:
        return None
    return dict((
        (fromjson(datatype.key_type, item[0]),
            fromjson(datatype.value_type, item[1]))
        for item in value.items()))


@fromjson.when_object(six.binary_type)
def str_fromjson(datatype, value):
    if (isinstance(value, six.string_types)
            or isinstance(value, six.integer_types)
            or isinstance(value, float)):
        return six.text_type(value).encode('utf8')


@fromjson.when_object(wsme.types.text)
def text_fromjson(datatype, value):
    if value is not None and isinstance(value, wsme.types.bytes):
        return wsme.types.text(value)
    return value


@fromjson.when_object(decimal.Decimal)
def decimal_fromjson(datatype, value):
    if value is None:
        return None
    return decimal.Decimal(value)


@fromjson.when_object(datetime.date)
def date_fromjson(datatype, value):
    if value is None:
        return None
    return wsme.utils.parse_isodate(value)


@fromjson.when_object(datetime.time)
def time_fromjson(datatype, value):
    if value is None:
        return None
    return wsme.utils.parse_isotime(value)


@fromjson.when_object(datetime.datetime)
def datetime_fromjson(datatype, value):
    if value is None:
        return None
    return wsme.utils.parse_isodatetime(value)


def parse(s, datatypes, bodyarg, encoding='utf8'):
    if hasattr(s, 'read'):
        jdata = json.load(s)
    else:
        if six.PY3 and isinstance(s, six.binary_type):
            s = s.decode(encoding)
        jdata = json.loads(s)
    if bodyarg:
        argname = list(datatypes.keys())[0]
        kw = {argname: fromjson(datatypes[argname], jdata)}
    else:
        kw = {}
        extra_args = []
        for key in jdata:
            if key not in datatypes:
                extra_args.append(key)
            else:
                kw[key] = fromjson(datatypes[key], jdata[key])
        if extra_args:
            raise UnknownArgument(', '.join(extra_args))
    return kw


def encode_result(value, datatype, **options):
    jsondata = tojson(datatype, value)
    if options.get('nest_result', False):
        jsondata = {options.get('nested_result_attrname', 'result'): jsondata}
    return json.dumps(jsondata)


def encode_error(context, errordetail):
    return json.dumps(errordetail)


def encode_sample_value(datatype, value, format=False):
    r = tojson(datatype, value)
    content = json.dumps(r, ensure_ascii=False, indent=4 if format else 0,
                         sort_keys=format)
    return ('javascript', content)


def encode_sample_params(params, format=False):
    kw = {}
    for name, datatype, value in params:
        kw[name] = tojson(datatype, value)
    content = json.dumps(kw, ensure_ascii=False, indent=4 if format else 0,
                         sort_keys=format)
    return ('javascript', content)


def encode_sample_result(datatype, value, format=False):
    r = tojson(datatype, value)
    content = json.dumps(r, ensure_ascii=False, indent=4 if format else 0,
                         sort_keys=format)
    return ('javascript', content)



from wsmeext.extdirect import datastore


class SADataStoreController(datastore.DataStoreController):
    __dbsession__ = None
    __datatype__ = None

    def read(self, query=None, sort=None, page=None, start=None, limit=None):
        q = self.__dbsession__.query(self.__datatype__.__saclass__)
        total = q.count()
        if start is not None and limit is not None:
            q = q.slice(start, limit)
        return self.__readresulttype__(
            data=[
                self.__datatype__(o) for o in q
            ],
            success=True,
            total=total
        )



"""
A SOAP implementation for wsme.
Parts of the code were taken from the tgwebservices soap implmentation.
"""
from __future__ import absolute_import

import pkg_resources
import datetime
import decimal
import base64
import logging

import six

from wsmeext.soap.simplegeneric import generic
from wsmeext.soap.wsdl import WSDLGenerator

try:
    from lxml import etree as ET
    use_lxml = True
except ImportError:
    from xml.etree import cElementTree as ET  # noqa
    use_lxml = False

from wsme.protocol import CallContext, Protocol, expose

import wsme.types
import wsme.runtime

from wsme import exc
from wsme.utils import parse_isodate, parse_isotime, parse_isodatetime

log = logging.getLogger(__name__)

xsd_ns = 'http://www.w3.org/2001/XMLSchema'
xsi_ns = 'http://www.w3.org/2001/XMLSchema-instance'
soapenv_ns = 'http://schemas.xmlsoap.org/soap/envelope/'

if not use_lxml:
    ET.register_namespace('soap', soapenv_ns)

type_qn = '{%s}type' % xsi_ns
nil_qn = '{%s}nil' % xsi_ns

Envelope_qn = '{%s}Envelope' % soapenv_ns
Body_qn = '{%s}Body' % soapenv_ns
Fault_qn = '{%s}Fault' % soapenv_ns
faultcode_qn = '{%s}faultcode' % soapenv_ns
faultstring_qn = '{%s}faultstring' % soapenv_ns
detail_qn = '{%s}detail' % soapenv_ns


type_registry = {
    wsme.types.bytes: 'xs:string',
    wsme.types.text: 'xs:string',
    int: 'xs:int',
    float: "xs:float",
    bool: "xs:boolean",
    datetime.datetime: "xs:dateTime",
    datetime.date: "xs:date",
    datetime.time: "xs:time",
    decimal.Decimal: "xs:decimal",
    wsme.types.binary: "xs:base64Binary",
}

if not six.PY3:
    type_registry[long] = "xs:long"

array_registry = {
    wsme.types.text: "String_Array",
    wsme.types.bytes: "String_Array",
    int: "Int_Array",
    float: "Float_Array",
    bool: "Boolean_Array",
}

if not six.PY3:
    array_registry[long] = "Long_Array"


def soap_array(datatype, ns):
    if datatype.item_type in array_registry:
        name = array_registry[datatype.item_type]
    else:
        name = soap_type(datatype.item_type, False) + '_Array'
    if ns:
        name = 'types:' + name
    return name


def soap_type(datatype, ns):
    name = None
    if wsme.types.isarray(datatype):
        return soap_array(datatype, ns)
    if wsme.types.isdict(datatype):
        return None
    if datatype in type_registry:
        stype = type_registry[datatype]
        if not ns:
            stype = stype[3:]
        return stype
    if wsme.types.iscomplex(datatype):
        name = datatype.__name__
        if name and ns:
            name = 'types:' + name
        return name
    if wsme.types.isusertype(datatype):
        return soap_type(datatype.basetype, ns)


def soap_fname(path, funcdef):
    return "".join([path[0]] + [i.capitalize() for i in path[1:]])


class SoapEncoder(object):
    def __init__(self, types_ns):
        self.types_ns = types_ns

    def make_soap_element(self, datatype, tag, value, xsitype=None):
        el = ET.Element(tag)
        if value is None:
            el.set(nil_qn, 'true')
        elif xsitype is not None:
            el.set(type_qn, xsitype)
            el.text = value
        elif wsme.types.isusertype(datatype):
            return self.tosoap(datatype.basetype, tag,
                               datatype.tobasetype(value))
        elif wsme.types.iscomplex(datatype):
            el.set(type_qn, 'types:%s' % (datatype.__name__))
            for attrdef in wsme.types.list_attributes(datatype):
                attrvalue = getattr(value, attrdef.key)
                if attrvalue is not wsme.types.Unset:
                    el.append(self.tosoap(
                        attrdef.datatype,
                        '{%s}%s' % (self.types_ns, attrdef.name),
                        attrvalue
                    ))
        else:
            el.set(type_qn, type_registry.get(datatype))
            if not isinstance(value, wsme.types.text):
                value = wsme.types.text(value)
            el.text = value
        return el

    @generic
    def tosoap(self, datatype, tag, value):
        """Converts a value into xml Element objects for inclusion in the SOAP
        response output (after adding the type to the type_registry).

        If a non-complex user specific type is to be used in the api,
        a specific toxml should be added::

            from wsme.protocol.soap import tosoap, make_soap_element, \
                type_registry

            class MySpecialType(object):
                pass

            type_registry[MySpecialType] = 'xs:MySpecialType'

            @tosoap.when_object(MySpecialType)
            def myspecialtype_tosoap(datatype, tag, value):
                return make_soap_element(datatype, tag, str(value))
        """
        return self.make_soap_element(datatype, tag, value)

    @tosoap.when_type(wsme.types.ArrayType)
    def array_tosoap(self, datatype, tag, value):
        el = ET.Element(tag)
        el.set(type_qn, soap_array(datatype, self.types_ns))
        if value is None:
            el.set(nil_qn, 'true')
        elif len(value) == 0:
            el.append(ET.Element('item'))
        else:
            for item in value:
                el.append(self.tosoap(datatype.item_type, 'item', item))
        return el

    @tosoap.when_object(bool)
    def bool_tosoap(self, datatype, tag, value):
        return self.make_soap_element(
            datatype,
            tag,
            'true' if value is True else 'false' if value is False else None
        )

    @tosoap.when_object(wsme.types.bytes)
    def bytes_tosoap(self, datatype, tag, value):
        print('bytes_tosoap', datatype, tag, value, type(value))
        if isinstance(value, wsme.types.bytes):
            value = value.decode('ascii')
        return self.make_soap_element(datatype, tag, value)

    @tosoap.when_object(datetime.datetime)
    def datetime_tosoap(self, datatype, tag, value):
        return self.make_soap_element(
            datatype,
            tag,
            value is not None and value.isoformat() or None
        )

    @tosoap.when_object(wsme.types.binary)
    def binary_tosoap(self, datatype, tag, value):
        print(datatype, tag, value)
        value = base64.encodestring(value) if value is not None else None
        if six.PY3:
            value = value.decode('ascii')
        return self.make_soap_element(
            datatype.basetype, tag, value, 'xs:base64Binary'
        )

    @tosoap.when_object(None)
    def None_tosoap(self, datatype, tag, value):
        return self.make_soap_element(datatype, tag, None)


@generic
def fromsoap(datatype, el, ns):
    """
    A generic converter from soap elements to python datatype.

    If a non-complex user specific type is to be used in the api,
    a specific fromsoap should be added.
    """
    if el.get(nil_qn) == 'true':
        return None
    if datatype in type_registry:
        value = datatype(el.text)
    elif wsme.types.isusertype(datatype):
        value = datatype.frombasetype(
            fromsoap(datatype.basetype, el, ns))
    else:
        value = datatype()
        for attr in wsme.types.list_attributes(datatype):
            child = el.find('{%s}%s' % (ns['type'], attr.name))
            if child is not None:
                setattr(value, attr.key, fromsoap(attr.datatype, child, ns))
    return value


@fromsoap.when_type(wsme.types.ArrayType)
def array_fromsoap(datatype, el, ns):
    if len(el) == 1:
        if datatype.item_type \
                not in wsme.types.pod_types + wsme.types.dt_types \
                and len(el[0]) == 0:
            return []
    return [fromsoap(datatype.item_type, child, ns) for child in el]


@fromsoap.when_object(wsme.types.bytes)
def bytes_fromsoap(datatype, el, ns):
    if el.get(nil_qn) == 'true':
        return None
    if el.get(type_qn) not in (None, 'xs:string'):
        raise exc.InvalidInput(el.tag, ET.tostring(el))
    return el.text.encode('ascii') if el.text else six.b('')


@fromsoap.when_object(wsme.types.text)
def text_fromsoap(datatype, el, ns):
    if el.get(nil_qn) == 'true':
        return None
    if el.get(type_qn) not in (None, 'xs:string'):
        raise exc.InvalidInput(el.tag, ET.tostring(el))
    return datatype(el.text if el.text else '')


@fromsoap.when_object(bool)
def bool_fromsoap(datatype, el, ns):
    if el.get(nil_qn) == 'true':
        return None
    if el.get(type_qn) not in (None, 'xs:boolean'):
        raise exc.InvalidInput(el.tag, ET.tostring(el))
    return el.text.lower() != 'false'


@fromsoap.when_object(datetime.date)
def date_fromsoap(datatype, el, ns):
    if el.get(nil_qn) == 'true':
        return None
    if el.get(type_qn) not in (None, 'xs:date'):
        raise exc.InvalidInput(el.tag, ET.tostring(el))
    return parse_isodate(el.text)


@fromsoap.when_object(datetime.time)
def time_fromsoap(datatype, el, ns):
    if el.get(nil_qn) == 'true':
        return None
    if el.get(type_qn) not in (None, 'xs:time'):
        raise exc.InvalidInput(el.tag, ET.tostring(el))
    return parse_isotime(el.text)


@fromsoap.when_object(datetime.datetime)
def datetime_fromsoap(datatype, el, ns):
    if el.get(nil_qn) == 'true':
        return None
    if el.get(type_qn) not in (None, 'xs:dateTime'):
        raise exc.InvalidInput(el.tag, ET.tostring(el))
    return parse_isodatetime(el.text)


@fromsoap.when_object(wsme.types.binary)
def binary_fromsoap(datatype, el, ns):
    if el.get(nil_qn) == 'true':
        return None
    if el.get(type_qn) not in (None, 'xs:base64Binary'):
        raise exc.InvalidInput(el.tag, ET.tostring(el))
    return base64.decodestring(el.text.encode('ascii'))


class SoapProtocol(Protocol):
    """
    SOAP protocol.

    .. autoattribute:: name
    .. autoattribute:: content_types
    """
    name = 'soap'
    displayname = 'SOAP'
    content_types = ['application/soap+xml']

    ns = {
        "soap": "http://www.w3.org/2001/12/soap-envelope",
        "soapenv": "http://schemas.xmlsoap.org/soap/envelope/",
        "soapenc": "http://schemas.xmlsoap.org/soap/encoding/",
    }

    def __init__(self, tns=None, typenamespace=None, baseURL=None,
                 servicename='MyApp'):
        self.tns = tns
        self.typenamespace = typenamespace
        self.servicename = servicename
        self.baseURL = baseURL
        self._name_mapping = {}

        self.encoder = SoapEncoder(typenamespace)

    def get_name_mapping(self, service=None):
        if service not in self._name_mapping:
            self._name_mapping[service] = dict(
                (soap_fname(path, f), path)
                for path, f in self.root.getapi()
                if service is None or (path and path[0] == service)
            )
        return self._name_mapping[service]

    def accept(self, req):
        for ct in self.content_types:
            if req.headers['Content-Type'].startswith(ct):
                return True
        if req.headers.get("Soapaction"):
            return True
        return False

    def iter_calls(self, request):
        yield CallContext(request)

    def extract_path(self, context):
        request = context.request
        el = ET.fromstring(request.body)
        body = el.find('{%(soapenv)s}Body' % self.ns)
        # Extract the service name from the tns
        message = list(body)[0]
        fname = message.tag
        if fname.startswith('{%s}' % self.typenamespace):
            fname = fname[len(self.typenamespace) + 2:]
            mapping = self.get_name_mapping()
            if fname not in mapping:
                raise exc.UnknownFunction(fname)
            path = mapping[fname]
            context.soap_message = message
            return path
        return None

    def read_arguments(self, context):
        kw = {}
        if not hasattr(context, 'soap_message'):
            return kw
        msg = context.soap_message
        for param in msg:
            name = param.tag[len(self.typenamespace) + 2:]
            arg = context.funcdef.get_arg(name)
            value = fromsoap(arg.datatype, param, {
                'type': self.typenamespace,
            })
            kw[name] = value
        wsme.runtime.check_arguments(context.funcdef, (), kw)
        return kw

    def soap_response(self, path, funcdef, result):
        r = ET.Element('{%s}%sResponse' % (
            self.typenamespace, soap_fname(path, funcdef)
        ))
        print('soap_response', funcdef.return_type, result)
        r.append(self.encoder.tosoap(
            funcdef.return_type, '{%s}result' % self.typenamespace, result
        ))
        return r

    def encode_result(self, context, result):
        print('encode_result', result)
        if use_lxml:
            envelope = ET.Element(
                Envelope_qn,
                nsmap={'xs': xsd_ns, 'types': self.typenamespace}
            )
        else:
            envelope = ET.Element(Envelope_qn, {
                'xmlns:xs': xsd_ns,
                'xmlns:types': self.typenamespace
            })
        body = ET.SubElement(envelope, Body_qn)
        body.append(self.soap_response(context.path, context.funcdef, result))
        s = ET.tostring(envelope)
        return s

    def get_template(self, name):
        return pkg_resources.resource_string(
            __name__, '%s.html' % name)

    def encode_error(self, context, infos):
        envelope = ET.Element(Envelope_qn)
        body = ET.SubElement(envelope, Body_qn)
        fault = ET.SubElement(body, Fault_qn)
        ET.SubElement(fault, faultcode_qn).text = infos['faultcode']
        ET.SubElement(fault, faultstring_qn).text = infos['faultstring']
        if 'debuginfo' in infos:
            ET.SubElement(fault, detail_qn).text = infos['debuginfo']
        s = ET.tostring(envelope)
        return s

    @expose('/api.wsdl', 'text/xml')
    def api_wsdl(self, service=None):
        if service is None:
            servicename = self.servicename
        else:
            servicename = self.servicename + service.capitalize()
        return WSDLGenerator(
            tns=self.tns,
            types_ns=self.typenamespace,
            soapenc=self.ns['soapenc'],
            service_name=servicename,
            complex_types=self.root.__registry__.complex_types,
            funclist=self.root.getapi(),
            arrays=self.root.__registry__.array_types,
            baseURL=self.baseURL,
            soap_array=soap_array,
            soap_type=soap_type,
            soap_fname=soap_fname,
        ).generate(True)

    def encode_sample_value(self, datatype, value, format=False):
        r = self.encoder.make_soap_element(datatype, 'value', value)
        if format:
            xml_indent(r)
        return ('xml', unicode(r))


def xml_indent(elem, level=0):
    i = "\n" + level * "  "
    if len(elem):
        if not elem.text or not elem.text.strip():
            elem.text = i + "  "
        for e in elem:
            xml_indent(e, level + 1)
        if not e.tail or not e.tail.strip():
            e.tail = i
    if level and (not elem.tail or not elem.tail.strip()):
        elem.tail = i



from __future__ import absolute_import

import datetime

import six

import xml.etree.ElementTree as et

from simplegeneric import generic

import wsme.types
from wsme.exc import UnknownArgument, InvalidInput

import re

content_type = 'text/xml'
accept_content_types = [
    content_type,
]

time_re = re.compile(r'(?P<h>[0-2][0-9]):(?P<m>[0-5][0-9]):(?P<s>[0-6][0-9])')


def xml_indent(elem, level=0):
    i = "\n" + level * "  "
    if len(elem):
        if not elem.text or not elem.text.strip():
            elem.text = i + "  "
        for e in elem:
            xml_indent(e, level + 1)
        if not e.tail or not e.tail.strip():
            e.tail = i
    if level and (not elem.tail or not elem.tail.strip()):
        elem.tail = i


@generic
def toxml(datatype, key, value):
    """
    A generic converter from python to xml elements.

    If a non-complex user specific type is to be used in the api,
    a specific toxml should be added::

        from wsme.protocol.restxml import toxml

        myspecialtype = object()

        @toxml.when_object(myspecialtype)
        def myspecialtype_toxml(datatype, key, value):
            el = et.Element(key)
            if value is None:
                el.set('nil', 'true')
            else:
                el.text = str(value)
            return el
    """
    el = et.Element(key)
    if value is None:
        el.set('nil', 'true')
    else:
        if wsme.types.isusertype(datatype):
            return toxml(datatype.basetype,
                         key, datatype.tobasetype(value))
        elif wsme.types.iscomplex(datatype):
            for attrdef in datatype._wsme_attributes:
                attrvalue = getattr(value, attrdef.key)
                if attrvalue is not wsme.types.Unset:
                    el.append(toxml(attrdef.datatype, attrdef.name,
                                    attrvalue))
        else:
            el.text = six.text_type(value)
    return el


@generic
def fromxml(datatype, element):
    """
    A generic converter from xml elements to python datatype.

    If a non-complex user specific type is to be used in the api,
    a specific fromxml should be added::

        from wsme.protocol.restxml import fromxml

        class MySpecialType(object):
            pass

        @fromxml.when_object(MySpecialType)
        def myspecialtype_fromxml(datatype, element):
            if element.get('nil', False):
                return None
            return MySpecialType(element.text)
    """
    if element.get('nil', False):
        return None
    if wsme.types.isusertype(datatype):
        return datatype.frombasetype(fromxml(datatype.basetype, element))
    if wsme.types.iscomplex(datatype):
        obj = datatype()
        for attrdef in wsme.types.list_attributes(datatype):
            sub = element.find(attrdef.name)
            if sub is not None:
                val_fromxml = fromxml(attrdef.datatype, sub)
                if getattr(attrdef, 'readonly', False):
                    raise InvalidInput(attrdef.name, val_fromxml,
                                       "Cannot set read only field.")
                setattr(obj, attrdef.key, val_fromxml)
            elif attrdef.mandatory:
                raise InvalidInput(attrdef.name, None,
                                   "Mandatory field missing.")
        return wsme.types.validate_value(datatype, obj)
    if datatype is wsme.types.bytes:
        return element.text.encode('ascii')
    return datatype(element.text)


@toxml.when_type(wsme.types.ArrayType)
def array_toxml(datatype, key, value):
    el = et.Element(key)
    if value is None:
        el.set('nil', 'true')
    else:
        for item in value:
            el.append(toxml(datatype.item_type, 'item', item))
    return el


@toxml.when_type(wsme.types.DictType)
def dict_toxml(datatype, key, value):
    el = et.Element(key)
    if value is None:
        el.set('nil', 'true')
    else:
        for item in value.items():
            key = toxml(datatype.key_type, 'key', item[0])
            value = toxml(datatype.value_type, 'value', item[1])
            node = et.Element('item')
            node.append(key)
            node.append(value)
            el.append(node)
    return el


@toxml.when_object(wsme.types.bytes)
def bytes_toxml(datatype, key, value):
    el = et.Element(key)
    if value is None:
        el.set('nil', 'true')
    else:
        el.text = value.decode('ascii')
    return el


@toxml.when_object(bool)
def bool_toxml(datatype, key, value):
    el = et.Element(key)
    if value is None:
        el.set('nil', 'true')
    else:
        el.text = value and 'true' or 'false'
    return el


@toxml.when_object(datetime.date)
def date_toxml(datatype, key, value):
    el = et.Element(key)
    if value is None:
        el.set('nil', 'true')
    else:
        el.text = value.isoformat()
    return el


@toxml.when_object(datetime.datetime)
def datetime_toxml(datatype, key, value):
    el = et.Element(key)
    if value is None:
        el.set('nil', 'true')
    else:
        el.text = value.isoformat()
    return el


@fromxml.when_type(wsme.types.ArrayType)
def array_fromxml(datatype, element):
    if element.get('nil') == 'true':
        return None
    return [
        fromxml(datatype.item_type, item)
        for item in element.findall('item')
    ]


@fromxml.when_object(bool)
def bool_fromxml(datatype, element):
    if element.get('nil') == 'true':
        return None
    return element.text.lower() != 'false'


@fromxml.when_type(wsme.types.DictType)
def dict_fromxml(datatype, element):
    if element.get('nil') == 'true':
        return None
    return dict((
        (fromxml(datatype.key_type, item.find('key')),
            fromxml(datatype.value_type, item.find('value')))
        for item in element.findall('item')))


@fromxml.when_object(wsme.types.text)
def unicode_fromxml(datatype, element):
    if element.get('nil') == 'true':
        return None
    return wsme.types.text(element.text) if element.text else six.u('')


@fromxml.when_object(datetime.date)
def date_fromxml(datatype, element):
    if element.get('nil') == 'true':
        return None
    return wsme.utils.parse_isodate(element.text)


@fromxml.when_object(datetime.time)
def time_fromxml(datatype, element):
    if element.get('nil') == 'true':
        return None
    return wsme.utils.parse_isotime(element.text)


@fromxml.when_object(datetime.datetime)
def datetime_fromxml(datatype, element):
    if element.get('nil') == 'true':
        return None
    return wsme.utils.parse_isodatetime(element.text)


def parse(s, datatypes, bodyarg):
    if hasattr(s, 'read'):
        tree = et.parse(s)
    else:
        tree = et.fromstring(s)
    if bodyarg:
        name = list(datatypes.keys())[0]
        return {name: fromxml(datatypes[name], tree)}
    else:
        kw = {}
        extra_args = []
        for sub in tree:
            if sub.tag not in datatypes:
                extra_args.append(sub.tag)
            kw[sub.tag] = fromxml(datatypes[sub.tag], sub)
        if extra_args:
            raise UnknownArgument(', '.join(extra_args))
        return kw


def encode_result(value, datatype, **options):
    return et.tostring(toxml(
        datatype, options.get('nested_result_attrname', 'result'), value
    ))


def encode_error(context, errordetail):
    el = et.Element('error')
    et.SubElement(el, 'faultcode').text = errordetail['faultcode']
    et.SubElement(el, 'faultstring').text = errordetail['faultstring']
    if 'debuginfo' in errordetail:
        et.SubElement(el, 'debuginfo').text = errordetail['debuginfo']
    return et.tostring(el)


def encode_sample_value(datatype, value, format=False):
    r = toxml(datatype, 'value', value)
    if format:
        xml_indent(r)
    content = et.tostring(r)
    return ('xml', content)


def encode_sample_params(params, format=False):
    node = et.Element('parameters')
    for name, datatype, value in params:
        node.append(toxml(datatype, name, value))
    if format:
        xml_indent(node)
    content = et.tostring(node)
    return ('xml', content)


def encode_sample_result(datatype, value, format=False):
    r = toxml(datatype, 'result', value)
    if format:
        xml_indent(r)
    content = et.tostring(r)
    return ('xml', content)



import unittest
import sphinx
import os.path

import wsme.types
from wsmeext import sphinxext

docpath = os.path.join(
    os.path.dirname(__file__),
    'sphinxexample')


class ASampleType(object):
    somebytes = wsme.types.bytes
    sometext = wsme.types.text
    someint = int


class TestSphinxExt(unittest.TestCase):
    def test_buildhtml(self):
        if not os.path.exists('.test_sphinxext/'):
            os.makedirs('.test_sphinxext/')
        assert sphinx.main(['',
            '-b', 'html',
            '-d', '.test_sphinxext/doctree',
            docpath,
            '.test_sphinxext/html']) == 0


class TestDataTypeName(unittest.TestCase):
    def test_user_type(self):
        self.assertEqual(sphinxext.datatypename(ASampleType),
                         'ASampleType')

    def test_dict_type(self):
        d = wsme.types.DictType(str, str)
        self.assertEqual(sphinxext.datatypename(d), 'dict(str: str)')
        d = wsme.types.DictType(str, ASampleType)
        self.assertEqual(sphinxext.datatypename(d), 'dict(str: ASampleType)')

    def test_array_type(self):
        d = wsme.types.ArrayType(str)
        self.assertEqual(sphinxext.datatypename(d), 'list(str)')
        d = wsme.types.ArrayType(ASampleType)
        self.assertEqual(sphinxext.datatypename(d), 'list(ASampleType)')



# Server Specific Configurations
server = {
    'port' : '8080',
    'host' : '0.0.0.0'
}

# Pecan Application Configurations
app = {
    'root'            : 'test.controllers.root.RootController',
    'modules'         : ['test'],
    'static_root'     : '%(confdir)s/../../public',
    'template_path'   : '%(confdir)s/../templates',
    'errors'          : {
        '404'            : '/error/404',
        '__force_dict__' : True
    }
}

# Custom Configurations must be in Python dictionary format::
#
# foo = {'bar':'baz'}
#
# All configurations are accessible at::
# pecan.conf



from turbogears import config
import cherrypy
from cherrypy.filters.basefilter import BaseFilter
from turbogears.startup import call_on_startup, call_on_shutdown
from wsmeext.tg1 import wsexpose, wsvalidate
import wsmeext.tg1

__all__ = ['adapt', 'wsexpose', 'wsvalidate']


class WSMECherrypyFilter(BaseFilter):
    def __init__(self, controller):
        self.controller = controller
        self.webpath = None

    def on_start_resource(self):
        path = cherrypy.request.path
        if path.startswith(self.controller._wsroot._webpath):
            cherrypy.request.processRequestBody = False


def adapt(wsroot):
    wsroot._scan_api = wsmeext.tg1.scan_api
    controller = wsmeext.tg1.Controller(wsroot)
    filter_ = WSMECherrypyFilter(controller)

    def install_filter():
        filter_.webpath = config.get('server.webpath') or ''
        controller._wsroot._webpath = \
            filter_.webpath + controller._wsroot._webpath
        cherrypy.root._cp_filters.append(filter_)

    def uninstall_filter():
        cherrypy.root._cp_filters.remove(filter_)
        controller._wsroot._webpath = \
            controller._wsroot._webpath[len(filter_.webpath):]

    call_on_startup.append(install_filter)
    call_on_shutdown.insert(0, uninstall_filter)
    return controller



from pecan import expose
from webob.exc import status_map
from .ws import AuthorsController
from wsmeext.pecan import wsexpose


class RootController(object):
    authors = AuthorsController()

    @expose('error.html')
    def error(self, status):
        try:
            status = int(status)
        except ValueError:  # pragma: no cover
            status = 500
        message = getattr(status_map.get(status), 'explanation', '')
        return dict(status=status, message=message)

    @wsexpose()
    def divide_by_zero(self):
        1 / 0



import re
try:
    import unittest2 as unittest
except ImportError:
    import unittest
import six

from wsme import exc
from wsme import types


def gen_class():
    d = {}
    exec('''class tmp(object): pass''', d)
    return d['tmp']


class TestTypes(unittest.TestCase):
    def setUp(self):
        types.registry = types.Registry()

    def test_default_usertype(self):
        class MyType(types.UserType):
            basetype = str

        My = MyType()

        assert My.validate('a') == 'a'
        assert My.tobasetype('a') == 'a'
        assert My.frombasetype('a') == 'a'

    def test_unset(self):
        u = types.Unset

        assert not u

    def test_flat_type(self):
        class Flat(object):
            aint = int
            abytes = six.binary_type
            atext = six.text_type
            afloat = float

        types.register_type(Flat)

        assert len(Flat._wsme_attributes) == 4
        attrs = Flat._wsme_attributes
        print(attrs)

        assert attrs[0].key == 'aint'
        assert attrs[0].name == 'aint'
        assert isinstance(attrs[0], types.wsattr)
        assert attrs[0].datatype == int
        assert attrs[0].mandatory is False
        assert attrs[1].key == 'abytes'
        assert attrs[1].name == 'abytes'
        assert attrs[2].key == 'atext'
        assert attrs[2].name == 'atext'
        assert attrs[3].key == 'afloat'
        assert attrs[3].name == 'afloat'

    def test_private_attr(self):
        class WithPrivateAttrs(object):
            _private = 12

        types.register_type(WithPrivateAttrs)

        assert len(WithPrivateAttrs._wsme_attributes) == 0

    def test_attribute_order(self):
        class ForcedOrder(object):
            _wsme_attr_order = ('a2', 'a1', 'a3')
            a1 = int
            a2 = int
            a3 = int

        types.register_type(ForcedOrder)

        print(ForcedOrder._wsme_attributes)
        assert ForcedOrder._wsme_attributes[0].key == 'a2'
        assert ForcedOrder._wsme_attributes[1].key == 'a1'
        assert ForcedOrder._wsme_attributes[2].key == 'a3'

        c = gen_class()
        print(c)
        types.register_type(c)
        del c._wsme_attributes

        c.a2 = int
        c.a1 = int
        c.a3 = int

        types.register_type(c)

        assert c._wsme_attributes[0].key == 'a1', c._wsme_attributes[0].key
        assert c._wsme_attributes[1].key == 'a2'
        assert c._wsme_attributes[2].key == 'a3'

    def test_wsproperty(self):
        class WithWSProp(object):
            def __init__(self):
                self._aint = 0

            def get_aint(self):
                return self._aint

            def set_aint(self, value):
                self._aint = value

            aint = types.wsproperty(int, get_aint, set_aint, mandatory=True)

        types.register_type(WithWSProp)

        print(WithWSProp._wsme_attributes)
        assert len(WithWSProp._wsme_attributes) == 1
        a = WithWSProp._wsme_attributes[0]
        assert a.key == 'aint'
        assert a.datatype == int
        assert a.mandatory

        o = WithWSProp()
        o.aint = 12

        assert o.aint == 12

    def test_nested(self):
        class Inner(object):
            aint = int

        class Outer(object):
            inner = Inner

        types.register_type(Outer)

        assert hasattr(Inner, '_wsme_attributes')
        assert len(Inner._wsme_attributes) == 1

    def test_inspect_with_inheritance(self):
        class Parent(object):
            parent_attribute = int

        class Child(Parent):
            child_attribute = int

        types.register_type(Parent)
        types.register_type(Child)

        assert len(Child._wsme_attributes) == 2

    def test_selfreftype(self):
        class SelfRefType(object):
            pass

        SelfRefType.parent = SelfRefType

        types.register_type(SelfRefType)

    def test_inspect_with_property(self):
        class AType(object):
            @property
            def test(self):
                return 'test'

        types.register_type(AType)

        assert len(AType._wsme_attributes) == 0
        assert AType().test == 'test'

    def test_enum(self):
        aenum = types.Enum(str, 'v1', 'v2')
        assert aenum.basetype is str

        class AType(object):
            a = aenum

        types.register_type(AType)

        assert AType.a.datatype is aenum

        obj = AType()
        obj.a = 'v1'
        assert obj.a == 'v1', repr(obj.a)

        self.assertRaisesRegexp(exc.InvalidInput,
                                "Invalid input for field/attribute a. \
Value: 'v3'. Value should be one of: v., v.",
                                setattr,
                                obj,
                                'a',
                                'v3')

    def test_attribute_validation(self):
        class AType(object):
            alist = [int]
            aint = int

        types.register_type(AType)

        obj = AType()

        obj.alist = [1, 2, 3]
        assert obj.alist == [1, 2, 3]
        obj.aint = 5
        assert obj.aint == 5

        self.assertRaises(exc.InvalidInput, setattr, obj, 'alist', 12)
        self.assertRaises(exc.InvalidInput, setattr, obj, 'alist', [2, 'a'])

    def test_text_attribute_conversion(self):
        class SType(object):
            atext = types.text
            abytes = types.bytes

        types.register_type(SType)

        obj = SType()

        obj.atext = six.b('somebytes')
        assert obj.atext == six.u('somebytes')
        assert isinstance(obj.atext, types.text)

        obj.abytes = six.u('sometext')
        assert obj.abytes == six.b('sometext')
        assert isinstance(obj.abytes, types.bytes)

    def test_named_attribute(self):
        class ABCDType(object):
            a_list = types.wsattr([int], name='a.list')
            astr = str

        types.register_type(ABCDType)

        assert len(ABCDType._wsme_attributes) == 2
        attrs = ABCDType._wsme_attributes

        assert attrs[0].key == 'a_list', attrs[0].key
        assert attrs[0].name == 'a.list', attrs[0].name
        assert attrs[1].key == 'astr', attrs[1].key
        assert attrs[1].name == 'astr', attrs[1].name

    def test_wsattr_del(self):
        class MyType(object):
            a = types.wsattr(int)

        types.register_type(MyType)

        value = MyType()

        value.a = 5
        assert value.a == 5
        del value.a
        assert value.a is types.Unset

    def test_validate_dict(self):
        assert types.validate_value({int: str}, {1: '1', 5: '5'})

        self.assertRaises(ValueError, types.validate_value,
                          {int: str}, [])

        assert types.validate_value({int: str}, {'1': '1', 5: '5'})

        self.assertRaises(ValueError, types.validate_value,
                          {int: str}, {1: 1, 5: '5'})

    def test_validate_list_valid(self):
        assert types.validate_value([int], [1, 2])
        assert types.validate_value([int], ['5'])

    def test_validate_list_empty(self):
        assert types.validate_value([int], []) == []

    def test_validate_list_none(self):
        v = types.ArrayType(int)
        assert v.validate(None) is None

    def test_validate_list_invalid_member(self):
        self.assertRaises(ValueError, types.validate_value, [int],
                          ['not-a-number'])

    def test_validate_list_invalid_type(self):
        self.assertRaises(ValueError, types.validate_value, [int], 1)

    def test_validate_float(self):
        self.assertEqual(types.validate_value(float, 1), 1.0)
        self.assertEqual(types.validate_value(float, '1'), 1.0)
        self.assertEqual(types.validate_value(float, 1.1), 1.1)
        self.assertRaises(ValueError, types.validate_value, float, [])
        self.assertRaises(ValueError, types.validate_value, float,
                          'not-a-float')

    def test_validate_int(self):
        self.assertEqual(types.validate_value(int, 1), 1)
        self.assertEqual(types.validate_value(int, '1'), 1)
        self.assertEqual(types.validate_value(int, six.u('1')), 1)
        self.assertRaises(ValueError, types.validate_value, int, 1.1)

    def test_validate_integer_type(self):
        v = types.IntegerType(minimum=1, maximum=10)
        v.validate(1)
        v.validate(5)
        v.validate(10)
        self.assertRaises(ValueError, v.validate, 0)
        self.assertRaises(ValueError, v.validate, 11)

    def test_validate_string_type(self):
        v = types.StringType(min_length=1, max_length=10,
                             pattern='^[a-zA-Z0-9]*$')
        v.validate('1')
        v.validate('12345')
        v.validate('1234567890')
        self.assertRaises(ValueError, v.validate, '')
        self.assertRaises(ValueError, v.validate, '12345678901')

        # Test a pattern validation
        v.validate('a')
        v.validate('A')
        self.assertRaises(ValueError, v.validate, '_')

    def test_validate_string_type_precompile(self):
        precompile = re.compile('^[a-zA-Z0-9]*$')
        v = types.StringType(min_length=1, max_length=10,
                             pattern=precompile)

        # Test a pattern validation
        v.validate('a')
        v.validate('A')
        self.assertRaises(ValueError, v.validate, '_')

    def test_validate_ipv4_address_type(self):
        v = types.IPv4AddressType()
        v.validate('127.0.0.1')
        v.validate('192.168.0.1')
        self.assertRaises(ValueError, v.validate, '')
        self.assertRaises(ValueError, v.validate, 'foo')
        self.assertRaises(ValueError, v.validate,
                          '2001:0db8:bd05:01d2:288a:1fc0:0001:10ee')

    def test_validate_ipv6_address_type(self):
        v = types.IPv6AddressType()
        v.validate('0:0:0:0:0:0:0:1')
        v.validate('2001:0db8:bd05:01d2:288a:1fc0:0001:10ee')
        self.assertRaises(ValueError, v.validate, '')
        self.assertRaises(ValueError, v.validate, 'foo')
        self.assertRaises(ValueError, v.validate, '192.168.0.1')

    def test_validate_uuid_type(self):
        v = types.UuidType()
        self.assertEqual(v.validate('6a0a707c-45ef-4758-b533-e55adddba8ce'),
                         '6a0a707c-45ef-4758-b533-e55adddba8ce')
        self.assertEqual(v.validate('6a0a707c45ef4758b533e55adddba8ce'),
                         '6a0a707c-45ef-4758-b533-e55adddba8ce')
        self.assertRaises(ValueError, v.validate, '')
        self.assertRaises(ValueError, v.validate, 'foo')
        self.assertRaises(ValueError, v.validate,
                          '6a0a707c-45ef-4758-b533-e55adddba8ce-a')

    def test_register_invalid_array(self):
        self.assertRaises(ValueError, types.register_type, [])
        self.assertRaises(ValueError, types.register_type, [int, str])
        self.assertRaises(AttributeError, types.register_type, [1])

    def test_register_invalid_dict(self):
        self.assertRaises(ValueError, types.register_type, {})
        self.assertRaises(ValueError, types.register_type,
                          {int: str, str: int})
        self.assertRaises(ValueError, types.register_type,
                          {types.Unset: str})

    def test_list_attribute_no_auto_register(self):
        class MyType(object):
            aint = int

        assert not hasattr(MyType, '_wsme_attributes')

        self.assertRaises(TypeError, types.list_attributes, MyType)

        assert not hasattr(MyType, '_wsme_attributes')

    def test_list_of_complextypes(self):
        class A(object):
            bs = types.wsattr(['B'])

        class B(object):
            i = int

        types.register_type(A)
        types.register_type(B)

        assert A.bs.datatype.item_type is B

    def test_cross_referenced_types(self):
        class A(object):
            b = types.wsattr('B')

        class B(object):
            a = A

        types.register_type(A)
        types.register_type(B)

        assert A.b.datatype is B

    def test_base(self):
        class B1(types.Base):
            b2 = types.wsattr('B2')

        class B2(types.Base):
            b2 = types.wsattr('B2')

        assert B1.b2.datatype is B2, repr(B1.b2.datatype)
        assert B2.b2.datatype is B2

    def test_base_init(self):
        class C1(types.Base):
            s = six.text_type

        c = C1(s=six.u('test'))
        assert c.s == six.u('test')

    def test_array_eq(self):
        l = [types.ArrayType(str)]
        assert types.ArrayType(str) in l

    def test_array_sample(self):
        s = types.ArrayType(str).sample()
        assert isinstance(s, list)
        assert s
        assert s[0] == ''

    def test_dict_sample(self):
        s = types.DictType(str, str).sample()
        assert isinstance(s, dict)
        assert s
        assert s == {'': ''}

    def test_binary_to_base(self):
        import base64
        assert types.binary.tobasetype(None) is None
        expected = base64.encodestring(six.b('abcdef'))
        assert types.binary.tobasetype(six.b('abcdef')) == expected

    def test_binary_from_base(self):
        import base64
        assert types.binary.frombasetype(None) is None
        encoded = base64.encodestring(six.b('abcdef'))
        assert types.binary.frombasetype(encoded) == six.b('abcdef')

    def test_wsattr_weakref_datatype(self):
        # If the datatype inside the wsattr ends up a weakref, it
        # should be converted to the real type when accessed again by
        # the property getter.
        import weakref
        a = types.wsattr(int)
        a.datatype = weakref.ref(int)
        assert a.datatype is int

    def test_wsattr_list_datatype(self):
        # If the datatype inside the wsattr ends up a list of weakrefs
        # to types, it should be converted to the real types when
        # accessed again by the property getter.
        import weakref
        a = types.wsattr(int)
        a.datatype = [weakref.ref(int)]
        assert isinstance(a.datatype, list)
        assert a.datatype[0] is int

    def test_file_get_content_by_reading(self):
        class buffer:
            def read(self):
                return 'abcdef'
        f = types.File(file=buffer())
        assert f.content == 'abcdef'

    def test_file_content_overrides_file(self):
        class buffer:
            def read(self):
                return 'from-file'
        f = types.File(content='from-content', file=buffer())
        assert f.content == 'from-content'

    def test_file_setting_content_discards_file(self):
        class buffer:
            def read(self):
                return 'from-file'
        f = types.File(file=buffer())
        f.content = 'from-content'
        assert f.content == 'from-content'

    def test_file_field_storage(self):
        class buffer:
            def read(self):
                return 'from-file'

        class fieldstorage:
            filename = 'static.json'
            file = buffer()
            type = 'application/json'
        f = types.File(fieldstorage=fieldstorage)
        assert f.content == 'from-file'

    def test_file_field_storage_value(self):
        class buffer:
            def read(self):
                return 'from-file'

        class fieldstorage:
            filename = 'static.json'
            file = None
            type = 'application/json'
            value = 'from-value'
        f = types.File(fieldstorage=fieldstorage)
        assert f.content == 'from-value'

    def test_file_property_file(self):
        class buffer:
            def read(self):
                return 'from-file'
        buf = buffer()
        f = types.File(file=buf)
        assert f.file is buf

    def test_file_property_content(self):
        class buffer:
            def read(self):
                return 'from-file'
        f = types.File(content=six.b('from-content'))
        assert f.file.read() == six.b('from-content')

    def test_unregister(self):
        class TempType(object):
            pass
        types.registry.register(TempType)
        v = types.registry.lookup('TempType')
        self.assertIs(v, TempType)
        types.registry._unregister(TempType)
        after = types.registry.lookup('TempType')
        self.assertIs(after, None)

    def test_unregister_twice(self):
        class TempType(object):
            pass
        types.registry.register(TempType)
        v = types.registry.lookup('TempType')
        self.assertIs(v, TempType)
        types.registry._unregister(TempType)
        # Second call should not raise an exception
        types.registry._unregister(TempType)
        after = types.registry.lookup('TempType')
        self.assertIs(after, None)

    def test_unregister_array_type(self):
        class TempType(object):
            pass
        t = [TempType]
        types.registry.register(t)
        self.assertNotEqual(types.registry.array_types, set())
        types.registry._unregister(t)
        self.assertEqual(types.registry.array_types, set())

    def test_unregister_array_type_twice(self):
        class TempType(object):
            pass
        t = [TempType]
        types.registry.register(t)
        self.assertNotEqual(types.registry.array_types, set())
        types.registry._unregister(t)
        # Second call should not raise an exception
        types.registry._unregister(t)
        self.assertEqual(types.registry.array_types, set())

    def test_unregister_dict_type(self):
        class TempType(object):
            pass
        t = {str: TempType}
        types.registry.register(t)
        self.assertNotEqual(types.registry.dict_types, set())
        types.registry._unregister(t)
        self.assertEqual(types.registry.dict_types, set())

    def test_unregister_dict_type_twice(self):
        class TempType(object):
            pass
        t = {str: TempType}
        types.registry.register(t)
        self.assertNotEqual(types.registry.dict_types, set())
        types.registry._unregister(t)
        # Second call should not raise an exception
        types.registry._unregister(t)
        self.assertEqual(types.registry.dict_types, set())

    def test_reregister(self):
        class TempType(object):
            pass
        types.registry.register(TempType)
        v = types.registry.lookup('TempType')
        self.assertIs(v, TempType)
        types.registry.reregister(TempType)
        after = types.registry.lookup('TempType')
        self.assertIs(after, TempType)

    def test_reregister_and_add_attr(self):
        class TempType(object):
            pass
        types.registry.register(TempType)
        attrs = types.list_attributes(TempType)
        self.assertEqual(attrs, [])
        TempType.one = str
        types.registry.reregister(TempType)
        after = types.list_attributes(TempType)
        self.assertNotEqual(after, [])

    def test_dynamicbase_add_attributes(self):
        class TempType(types.DynamicBase):
            pass
        types.registry.register(TempType)
        attrs = types.list_attributes(TempType)
        self.assertEqual(attrs, [])
        TempType.add_attributes(one=str)
        after = types.list_attributes(TempType)
        self.assertEqual(len(after), 1)

    def test_dynamicbase_add_attributes_second(self):
        class TempType(types.DynamicBase):
            pass
        types.registry.register(TempType)
        attrs = types.list_attributes(TempType)
        self.assertEqual(attrs, [])
        TempType.add_attributes(one=str)
        TempType.add_attributes(two=int)
        after = types.list_attributes(TempType)
        self.assertEqual(len(after), 2)



# coding=utf-8

import unittest
import warnings
import datetime
import decimal
import six

from six import u, b

from webtest import TestApp

from wsme import WSRoot, Unset
from wsme import expose, validate
import wsme.types
import wsme.utils

warnings.filterwarnings('ignore', module='webob.dec')

binarysample = b('\x00\xff\x43')

try:
    1 / 0
except ZeroDivisionError as e:
    zerodivisionerrormsg = str(e)


class CallException(RuntimeError):
    def __init__(self, faultcode, faultstring, debuginfo):
        self.faultcode = faultcode
        self.faultstring = faultstring
        self.debuginfo = debuginfo

    def __str__(self):
        return 'faultcode=%s, faultstring=%s, debuginfo=%s' % (
            self.faultcode, self.faultstring, self.debuginfo
        )


myenumtype = wsme.types.Enum(wsme.types.bytes, 'v1', 'v2')


class NestedInner(object):
    aint = int

    def __init__(self, aint=None):
        self.aint = aint


class NestedOuter(object):
    inner = NestedInner
    inner_array = wsme.types.wsattr([NestedInner])
    inner_dict = {wsme.types.text: NestedInner}

    def __init__(self):
        self.inner = NestedInner(0)


class NamedAttrsObject(object):
    def __init__(self, v1=Unset, v2=Unset):
        self.attr_1 = v1
        self.attr_2 = v2

    attr_1 = wsme.types.wsattr(int, name='attr.1')
    attr_2 = wsme.types.wsattr(int, name='attr.2')


class NestedInnerApi(object):
    @expose(bool)
    def deepfunction(self):
        return True


class NestedOuterApi(object):
    inner = NestedInnerApi()


class ReturnTypes(object):
    @expose(wsme.types.bytes)
    def getbytes(self):
        return b("astring")

    @expose(wsme.types.text)
    def gettext(self):
        return u('\xe3\x81\xae')

    @expose(int)
    def getint(self):
        return 2

    @expose(float)
    def getfloat(self):
        return 3.14159265

    @expose(decimal.Decimal)
    def getdecimal(self):
        return decimal.Decimal('3.14159265')

    @expose(datetime.date)
    def getdate(self):
        return datetime.date(1994, 1, 26)

    @expose(bool)
    def getbooltrue(self):
        return True

    @expose(bool)
    def getboolfalse(self):
        return False

    @expose(datetime.time)
    def gettime(self):
        return datetime.time(12, 0, 0)

    @expose(datetime.datetime)
    def getdatetime(self):
        return datetime.datetime(1994, 1, 26, 12, 0, 0)

    @expose(wsme.types.binary)
    def getbinary(self):
        return binarysample

    @expose(NestedOuter)
    def getnested(self):
        n = NestedOuter()
        return n

    @expose([wsme.types.bytes])
    def getbytesarray(self):
        return [b("A"), b("B"), b("C")]

    @expose([NestedOuter])
    def getnestedarray(self):
        return [NestedOuter(), NestedOuter()]

    @expose({wsme.types.bytes: NestedOuter})
    def getnesteddict(self):
        return {b('a'): NestedOuter(), b('b'): NestedOuter()}

    @expose(NestedOuter)
    def getobjectarrayattribute(self):
        obj = NestedOuter()
        obj.inner_array = [NestedInner(12), NestedInner(13)]
        return obj

    @expose(NestedOuter)
    def getobjectdictattribute(self):
        obj = NestedOuter()
        obj.inner_dict = {
            '12': NestedInner(12),
            '13': NestedInner(13)
        }
        return obj

    @expose(myenumtype)
    def getenum(self):
        return b('v2')

    @expose(NamedAttrsObject)
    def getnamedattrsobj(self):
        return NamedAttrsObject(5, 6)


class ArgTypes(object):
    def assertEquals(self, a, b):
        if not (a == b):
            raise AssertionError('%s != %s' % (a, b))

    @expose(wsme.types.bytes)
    @validate(wsme.types.bytes)
    def setbytes(self, value):
        print(repr(value))
        self.assertEquals(type(value), wsme.types.bytes)
        return value

    @expose(wsme.types.text)
    @validate(wsme.types.text)
    def settext(self, value):
        print(repr(value))
        self.assertEquals(type(value), wsme.types.text)
        return value

    @expose(wsme.types.text)
    @validate(wsme.types.text)
    def settextnone(self, value):
        print(repr(value))
        self.assertEquals(type(value), type(None))
        return value

    @expose(bool)
    @validate(bool)
    def setbool(self, value):
        print(repr(value))
        self.assertEquals(type(value), bool)
        return value

    @expose(int)
    @validate(int)
    def setint(self, value):
        print(repr(value))
        self.assertEquals(type(value), int)
        return value

    @expose(float)
    @validate(float)
    def setfloat(self, value):
        print(repr(value))
        self.assertEquals(type(value), float)
        return value

    @expose(decimal.Decimal)
    @validate(decimal.Decimal)
    def setdecimal(self, value):
        print(repr(value))
        self.assertEquals(type(value), decimal.Decimal)
        return value

    @expose(datetime.date)
    @validate(datetime.date)
    def setdate(self, value):
        print(repr(value))
        self.assertEquals(type(value), datetime.date)
        return value

    @expose(datetime.time)
    @validate(datetime.time)
    def settime(self, value):
        print(repr(value))
        self.assertEquals(type(value), datetime.time)
        return value

    @expose(datetime.datetime)
    @validate(datetime.datetime)
    def setdatetime(self, value):
        print(repr(value))
        self.assertEquals(type(value), datetime.datetime)
        return value

    @expose(wsme.types.binary)
    @validate(wsme.types.binary)
    def setbinary(self, value):
        print(repr(value))
        self.assertEquals(type(value), six.binary_type)
        return value

    @expose([wsme.types.bytes])
    @validate([wsme.types.bytes])
    def setbytesarray(self, value):
        print(repr(value))
        self.assertEquals(type(value), list)
        self.assertEquals(type(value[0]), wsme.types.bytes)
        return value

    @expose([wsme.types.text])
    @validate([wsme.types.text])
    def settextarray(self, value):
        print(repr(value))
        self.assertEquals(type(value), list)
        self.assertEquals(type(value[0]), wsme.types.text)
        return value

    @expose([datetime.datetime])
    @validate([datetime.datetime])
    def setdatetimearray(self, value):
        print(repr(value))
        self.assertEquals(type(value), list)
        self.assertEquals(type(value[0]), datetime.datetime)
        return value

    @expose(NestedOuter)
    @validate(NestedOuter)
    def setnested(self, value):
        print(repr(value))
        self.assertEquals(type(value), NestedOuter)
        return value

    @expose([NestedOuter])
    @validate([NestedOuter])
    def setnestedarray(self, value):
        print(repr(value))
        self.assertEquals(type(value), list)
        self.assertEquals(type(value[0]), NestedOuter)
        return value

    @expose({wsme.types.bytes: NestedOuter})
    @validate({wsme.types.bytes: NestedOuter})
    def setnesteddict(self, value):
        print(repr(value))
        self.assertEquals(type(value), dict)
        self.assertEquals(type(list(value.keys())[0]), wsme.types.bytes)
        self.assertEquals(type(list(value.values())[0]), NestedOuter)
        return value

    @expose(myenumtype)
    @validate(myenumtype)
    def setenum(self, value):
        print(value)
        self.assertEquals(type(value), wsme.types.bytes)
        return value

    @expose(NamedAttrsObject)
    @validate(NamedAttrsObject)
    def setnamedattrsobj(self, value):
        print(value)
        self.assertEquals(type(value), NamedAttrsObject)
        self.assertEquals(value.attr_1, 10)
        self.assertEquals(value.attr_2, 20)
        return value


class BodyTypes(object):
    def assertEquals(self, a, b):
        if not (a == b):
            raise AssertionError('%s != %s' % (a, b))

    @expose(int, body={wsme.types.text: int})
    @validate(int)
    def setdict(self, body):
        print(body)
        self.assertEquals(type(body), dict)
        self.assertEquals(type(body['test']), int)
        self.assertEquals(body['test'], 10)
        return body['test']

    @expose(int, body=[int])
    @validate(int)
    def setlist(self, body):
        print(body)
        self.assertEquals(type(body), list)
        self.assertEquals(type(body[0]), int)
        self.assertEquals(body[0], 10)
        return body[0]


class WithErrors(object):
    @expose()
    def divide_by_zero(self):
        1 / 0


class MiscFunctions(object):
    @expose(int)
    @validate(int, int)
    def multiply(self, a, b):
        return a * b


class WSTestRoot(WSRoot):
    argtypes = ArgTypes()
    returntypes = ReturnTypes()
    bodytypes = BodyTypes()
    witherrors = WithErrors()
    nested = NestedOuterApi()
    misc = MiscFunctions()

    def reset(self):
        self._touched = False

    @expose()
    def touch(self):
        self._touched = True


class ProtocolTestCase(unittest.TestCase):
    protocol_options = {}

    def assertTypedEquals(self, a, b, convert):
        if isinstance(a, six.string_types):
            a = convert(a)
        if isinstance(b, six.string_types):
            b = convert(b)
        self.assertEquals(a, b)

    def assertDateEquals(self, a, b):
        self.assertTypedEquals(a, b, wsme.utils.parse_isodate)

    def assertTimeEquals(self, a, b):
        self.assertTypedEquals(a, b, wsme.utils.parse_isotime)

    def assertDateTimeEquals(self, a, b):
        self.assertTypedEquals(a, b, wsme.utils.parse_isodatetime)

    def assertIntEquals(self, a, b):
        self.assertTypedEquals(a, b, int)

    def assertFloatEquals(self, a, b):
        self.assertTypedEquals(a, b, float)

    def assertDecimalEquals(self, a, b):
        self.assertTypedEquals(a, b, decimal.Decimal)

    def setUp(self):
        if self.__class__.__name__ != 'ProtocolTestCase':
            self.root = WSTestRoot()
            self.root.getapi()
            self.root.addprotocol(self.protocol, **self.protocol_options)

            self.app = TestApp(self.root.wsgiapp())

    def test_invalid_path(self):
        try:
            res = self.call('invalid_function')
            print(res)
            assert "No error raised"
        except CallException as e:
            self.assertEquals(e.faultcode, 'Client')
            self.assertEquals(e.faultstring.lower(),
                              u('unknown function name: invalid_function'))

    def test_serverside_error(self):
        try:
            res = self.call('witherrors/divide_by_zero')
            print(res)
            assert "No error raised"
        except CallException as e:
            self.assertEquals(e.faultcode, 'Server')
            self.assertEquals(e.faultstring, zerodivisionerrormsg)
            assert e.debuginfo is not None

    def test_serverside_error_nodebug(self):
        self.root._debug = False
        try:
            res = self.call('witherrors/divide_by_zero')
            print(res)
            assert "No error raised"
        except CallException as e:
            self.assertEquals(e.faultcode, 'Server')
            self.assertEquals(e.faultstring, zerodivisionerrormsg)
            assert e.debuginfo is None

    def test_touch(self):
        r = self.call('touch')
        assert r is None, r

    def test_return_bytes(self):
        r = self.call('returntypes/getbytes', _rt=wsme.types.bytes)
        self.assertEquals(r, b('astring'))

    def test_return_text(self):
        r = self.call('returntypes/gettext', _rt=wsme.types.text)
        self.assertEquals(r, u('\xe3\x81\xae'))

    def test_return_int(self):
        r = self.call('returntypes/getint')
        self.assertIntEquals(r, 2)

    def test_return_float(self):
        r = self.call('returntypes/getfloat')
        self.assertFloatEquals(r, 3.14159265)

    def test_return_decimal(self):
        r = self.call('returntypes/getdecimal')
        self.assertDecimalEquals(r, '3.14159265')

    def test_return_bool_true(self):
        r = self.call('returntypes/getbooltrue', _rt=bool)
        assert r

    def test_return_bool_false(self):
        r = self.call('returntypes/getboolfalse', _rt=bool)
        assert not r

    def test_return_date(self):
        r = self.call('returntypes/getdate')
        self.assertDateEquals(r, datetime.date(1994, 1, 26))

    def test_return_time(self):
        r = self.call('returntypes/gettime')
        self.assertTimeEquals(r, datetime.time(12))

    def test_return_datetime(self):
        r = self.call('returntypes/getdatetime')
        self.assertDateTimeEquals(r, datetime.datetime(1994, 1, 26, 12))

    def test_return_binary(self):
        r = self.call('returntypes/getbinary', _rt=wsme.types.binary)
        self.assertEquals(r, binarysample)

    def test_return_nested(self):
        r = self.call('returntypes/getnested', _rt=NestedOuter)
        self.assertEquals(r, {'inner': {'aint': 0}})

    def test_return_bytesarray(self):
        r = self.call('returntypes/getbytesarray', _rt=[six.binary_type])
        self.assertEquals(r, [b('A'), b('B'), b('C')])

    def test_return_nestedarray(self):
        r = self.call('returntypes/getnestedarray', _rt=[NestedOuter])
        self.assertEquals(r, [{'inner': {'aint': 0}}, {'inner': {'aint': 0}}])

    def test_return_nesteddict(self):
        r = self.call('returntypes/getnesteddict',
                      _rt={wsme.types.bytes: NestedOuter})
        self.assertEquals(r, {
            b('a'): {'inner': {'aint': 0}},
            b('b'): {'inner': {'aint': 0}}
        })

    def test_return_objectarrayattribute(self):
        r = self.call('returntypes/getobjectarrayattribute', _rt=NestedOuter)
        self.assertEquals(r, {
            'inner': {'aint': 0},
            'inner_array': [{'aint': 12}, {'aint': 13}]
        })

    def test_return_objectdictattribute(self):
        r = self.call('returntypes/getobjectdictattribute', _rt=NestedOuter)
        self.assertEquals(r, {
            'inner': {'aint': 0},
            'inner_dict': {
                '12': {'aint': 12},
                '13': {'aint': 13}
            }
        })

    def test_return_enum(self):
        r = self.call('returntypes/getenum', _rt=myenumtype)
        self.assertEquals(r, b('v2'), r)

    def test_return_namedattrsobj(self):
        r = self.call('returntypes/getnamedattrsobj', _rt=NamedAttrsObject)
        self.assertEquals(r, {'attr.1': 5, 'attr.2': 6})

    def test_setbytes(self):
        assert self.call('argtypes/setbytes', value=b('astring'),
                         _rt=wsme.types.bytes) == b('astring')

    def test_settext(self):
        assert self.call('argtypes/settext', value=u('\xe3\x81\xae'),
                         _rt=wsme.types.text) == u('\xe3\x81\xae')

    def test_settext_empty(self):
        assert self.call('argtypes/settext', value=u(''),
                         _rt=wsme.types.text) == u('')

    def test_settext_none(self):
        self.assertEquals(
            None,
            self.call('argtypes/settextnone', value=None, _rt=wsme.types.text)
        )

    def test_setint(self):
        r = self.call('argtypes/setint', value=3, _rt=int)
        self.assertEquals(r, 3)

    def test_setfloat(self):
        assert self.call('argtypes/setfloat', value=3.54,
                         _rt=float) == 3.54

    def test_setbool_true(self):
        r = self.call('argtypes/setbool', value=True, _rt=bool)
        assert r

    def test_setbool_false(self):
        r = self.call('argtypes/setbool', value=False, _rt=bool)
        assert not r

    def test_setdecimal(self):
        value = decimal.Decimal('3.14')
        assert self.call('argtypes/setdecimal', value=value,
                         _rt=decimal.Decimal) == value

    def test_setdate(self):
        value = datetime.date(2008, 4, 6)
        r = self.call('argtypes/setdate', value=value,
                      _rt=datetime.date)
        self.assertEquals(r, value)

    def test_settime(self):
        value = datetime.time(12, 12, 15)
        r = self.call('argtypes/settime', value=value,
                      _rt=datetime.time)
        self.assertEquals(r, datetime.time(12, 12, 15))

    def test_setdatetime(self):
        value = datetime.datetime(2008, 4, 6, 12, 12, 15)
        r = self.call('argtypes/setdatetime', value=value,
                      _rt=datetime.datetime)
        self.assertEquals(r, datetime.datetime(2008, 4, 6, 12, 12, 15))

    def test_setbinary(self):
        value = binarysample
        r = self.call('argtypes/setbinary', value=(value, wsme.types.binary),
                      _rt=wsme.types.binary) == value
        print(r)

    def test_setnested(self):
        value = {'inner': {'aint': 54}}
        r = self.call('argtypes/setnested',
                      value=(value, NestedOuter),
                      _rt=NestedOuter)
        self.assertEquals(r, value)

    def test_setnested_nullobj(self):
        value = {'inner': None}
        r = self.call(
            'argtypes/setnested',
            value=(value, NestedOuter),
            _rt=NestedOuter
        )
        self.assertEquals(r, value)

    def test_setbytesarray(self):
        value = [b("1"), b("2"), b("three")]
        r = self.call('argtypes/setbytesarray',
                      value=(value, [wsme.types.bytes]),
                      _rt=[wsme.types.bytes])
        self.assertEquals(r, value)

    def test_settextarray(self):
        value = [u("1")]
        r = self.call('argtypes/settextarray',
                      value=(value, [wsme.types.text]),
                      _rt=[wsme.types.text])
        self.assertEquals(r, value)

    def test_setdatetimearray(self):
        value = [
            datetime.datetime(2008, 3, 6, 12, 12, 15),
            datetime.datetime(2008, 4, 6, 2, 12, 15),
        ]
        r = self.call('argtypes/setdatetimearray',
                      value=(value, [datetime.datetime]),
                      _rt=[datetime.datetime])
        self.assertEquals(r, value)

    def test_setnestedarray(self):
        value = [
            {'inner': {'aint': 54}},
            {'inner': {'aint': 55}},
        ]
        r = self.call('argtypes/setnestedarray',
                      value=(value, [NestedOuter]),
                      _rt=[NestedOuter])
        self.assertEquals(r, value)

    def test_setnesteddict(self):
        value = {
            b('o1'): {'inner': {'aint': 54}},
            b('o2'): {'inner': {'aint': 55}},
        }
        r = self.call('argtypes/setnesteddict',
                      value=(value, {six.binary_type: NestedOuter}),
                      _rt={six.binary_type: NestedOuter})
        print(r)
        self.assertEquals(r, value)

    def test_setenum(self):
        value = b('v1')
        r = self.call('argtypes/setenum', value=value,
                      _rt=myenumtype)
        self.assertEquals(r, value)

    def test_setnamedattrsobj(self):
        value = {'attr.1': 10, 'attr.2': 20}
        r = self.call('argtypes/setnamedattrsobj',
                      value=(value, NamedAttrsObject),
                      _rt=NamedAttrsObject)
        self.assertEquals(r, value)

    def test_nested_api(self):
        r = self.call('nested/inner/deepfunction', _rt=bool)
        assert r is True

    def test_missing_argument(self):
        try:
            r = self.call('argtypes/setdatetime')
            print(r)
            assert "No error raised"
        except CallException as e:
            self.assertEquals(e.faultcode, 'Client')
            self.assertEquals(e.faultstring, u('Missing argument: "value"'))

    def test_misc_multiply(self):
        self.assertEquals(self.call('misc/multiply', a=5, b=2, _rt=int), 10)

    def test_html_format(self):
        res = self.call('argtypes/setdatetime', _accept="text/html",
                        _no_result_decode=True)
        self.assertEquals(res.content_type, 'text/html')


class RestOnlyProtocolTestCase(ProtocolTestCase):
    def test_body_list(self):
        r = self.call('bodytypes/setlist', body=([10], [int]), _rt=int)
        self.assertEqual(r, 10)

    def test_body_dict(self):
        r = self.call('bodytypes/setdict',
                      body=({'test': 10}, {wsme.types.text: int}),
                      _rt=int)
        self.assertEqual(r, 10)



import wsmeext.tg15
from wsme import WSRoot

from turbogears.controllers import RootController
import cherrypy

from wsmeext.tests import test_soap

import simplejson


class Subcontroller(object):
    @wsmeext.tg15.wsexpose(int, int, int)
    def add(self, a, b):
        return a + b


class Root(RootController):
    class UselessSubClass:
        # This class is here only to make sure wsmeext.tg1.scan_api
        # does its job properly
        pass

    sub = Subcontroller()

    ws = WSRoot(webpath='/ws')
    ws.addprotocol('soap',
        tns=test_soap.tns,
        typenamespace=test_soap.typenamespace,
        baseURL='/ws/'
    )
    ws = wsmeext.tg15.adapt(ws)

    @wsmeext.tg15.wsexpose(int)
    @wsmeext.tg15.wsvalidate(int, int)
    def multiply(self, a, b):
        return a * b

    @wsmeext.tg15.wsexpose(int)
    @wsmeext.tg15.wsvalidate(int, int)
    def divide(self, a, b):
        if b == 0:
            raise cherrypy.HTTPError(400, 'Cannot divide by zero!')
        return a / b


from turbogears import testutil


class TestController(testutil.TGTest):
    root = Root

#    def setUp(self):
#        "Tests the output of the index method"
#        self.app = testutil.make_app(self.root)
#        #print cherrypy.root
#        testutil.start_server()

#    def tearDown(self):
#        # implementation copied from turbogears.testutil.stop_server.
#        # The only change is that cherrypy.root is set to None
#        # AFTER stopTurbogears has been called so that wsmeext.tg15
#        # can correctly uninstall its filter.
#        if config.get("cp_started"):
#            cherrypy.server.stop()
#            config.update({"cp_started": False})
#
#        if config.get("server_started"):
#            startup.stopTurboGears()
#            config.update({"server_started": False})

    def test_restcall(self):
        response = self.app.post("/multiply",
            simplejson.dumps({'a': 5, 'b': 10}),
            {'Content-Type': 'application/json'}
        )
        print response
        assert simplejson.loads(response.body) == 50

        response = self.app.post("/multiply",
            simplejson.dumps({'a': 5, 'b': 10}),
            {'Content-Type': 'application/json', 'Accept': 'application/json'}
        )
        print response
        assert simplejson.loads(response.body) == 50

        response = self.app.post("/multiply",
            simplejson.dumps({'a': 5, 'b': 10}),
            {'Content-Type': 'application/json', 'Accept': 'text/javascript'}
        )
        print response
        assert simplejson.loads(response.body) == 50

        response = self.app.post("/multiply",
            simplejson.dumps({'a': 5, 'b': 10}),
            {'Content-Type': 'application/json',
             'Accept': 'text/xml'}
        )
        print response
        assert response.body == "<result>50</result>"

    def test_custom_clientside_error(self):
        response = self.app.post(
            "/divide",
            simplejson.dumps({'a': 5, 'b': 0}),
            {'Content-Type': 'application/json', 'Accept': 'application/json'},
            expect_errors=True
        )
        assert response.status_int == 400
        assert simplejson.loads(response.body) == {
            "debuginfo": None,
            "faultcode": "Client",
            "faultstring": "(400, 'Cannot divide by zero!')"
        }

        response = self.app.post(
            "/divide",
            simplejson.dumps({'a': 5, 'b': 0}),
            {'Content-Type': 'application/json', 'Accept': 'text/xml'},
            expect_errors=True
        )
        assert response.status_int == 400
        assert response.body == ("<error><faultcode>Client</faultcode>"
                                 "<faultstring>(400, 'Cannot divide by zero!')"
                                 "</faultstring><debuginfo /></error>")

    def test_soap_wsdl(self):
        wsdl = self.app.get('/ws/api.wsdl').body
        print wsdl
        assert 'multiply' in wsdl

    def test_soap_call(self):
        ts = test_soap.TestSOAP('test_wsdl')
        ts.app = self.app
        ts.ws_path = '/ws/'

        print ts.ws_path
        assert ts.call('multiply', a=5, b=10, _rt=int) == 50

    def test_scan_api_loops(self):
        class MyRoot(object):
            pass

        MyRoot.loop = MyRoot()

        root = MyRoot()

        api = list(wsmeext.tg1._scan_api(root))
        print(api)

        self.assertEquals(len(api), 0)

    def test_scan_api_maxlen(self):
        class ARoot(object):
            pass

        def make_subcontrollers(n):
            c = type('Controller%s' % n, (object,), {})
            return c

        c = ARoot
        for n in xrange(55):
            subc = make_subcontrollers(n)
            c.sub = subc()
            c = subc
        root = ARoot()
        self.assertRaises(ValueError, list, wsmeext.tg1._scan_api(root))

    def test_templates_content_type(self):
        self.assertEquals(
            "application/json",
            wsmeext.tg1.AutoJSONTemplate().get_content_type('dummy')
        )
        self.assertEquals(
            "text/xml",
            wsmeext.tg1.AutoXMLTemplate().get_content_type('dummy')
        )



# encoding=utf8

from six import b

try:
    import unittest2 as unittest
except ImportError:
    import unittest
import webtest

from wsme import WSRoot, expose, validate
from wsme.rest import scan_api
from wsme.api import FunctionArgument, FunctionDefinition
from wsme import types
import wsme.types

from wsme.tests.test_protocols import DummyProtocol


class TestController(unittest.TestCase):
    def test_expose(self):
        class MyWS(WSRoot):
            @expose(int)
            def getint(self):
                return 1

        assert MyWS.getint._wsme_definition.return_type == int

    def test_validate(self):
        class ComplexType(object):
            attr = int

        class MyWS(object):
            @expose(int)
            @validate(int, int, int)
            def add(self, a, b, c=0):
                return a + b + c

            @expose(bool)
            @validate(ComplexType)
            def setcplx(self, obj):
                pass

        MyWS.add._wsme_definition.resolve_types(wsme.types.registry)
        MyWS.setcplx._wsme_definition.resolve_types(wsme.types.registry)
        args = MyWS.add._wsme_definition.arguments

        assert args[0].name == 'a'
        assert args[0].datatype == int
        assert args[0].mandatory
        assert args[0].default is None

        assert args[1].name == 'b'
        assert args[1].datatype == int
        assert args[1].mandatory
        assert args[1].default is None

        assert args[2].name == 'c'
        assert args[2].datatype == int
        assert not args[2].mandatory
        assert args[2].default == 0

        assert types.iscomplex(ComplexType)

    def test_validate_enum_with_none(self):
        class Version(object):
            number = types.Enum(str, 'v1', 'v2', None)

        class MyWS(WSRoot):
            @expose(str)
            @validate(Version)
            def setcplx(self, version):
                pass

        r = MyWS(['restjson'])
        app = webtest.TestApp(r.wsgiapp())
        res = app.post_json('/setcplx', params={'version': {'number': 'arf'}},
                            expect_errors=True,
                            headers={'Accept': 'application/json'})
        self.assertTrue(
            res.json_body['faultstring'].startswith(
                "Invalid input for field/attribute number. Value: 'arf'. \
Value should be one of:"))
        self.assertIn('v1', res.json_body['faultstring'])
        self.assertIn('v2', res.json_body['faultstring'])
        self.assertIn('None', res.json_body['faultstring'])
        self.assertEqual(res.status_int, 400)

    def test_validate_enum_with_wrong_type(self):
        class Version(object):
            number = types.Enum(str, 'v1', 'v2', None)

        class MyWS(WSRoot):
            @expose(str)
            @validate(Version)
            def setcplx(self, version):
                pass

        r = MyWS(['restjson'])
        app = webtest.TestApp(r.wsgiapp())
        res = app.post_json('/setcplx', params={'version': {'number': 1}},
                            expect_errors=True,
                            headers={'Accept': 'application/json'})
        self.assertTrue(
            res.json_body['faultstring'].startswith(
                "Invalid input for field/attribute number. Value: '1'. \
Value should be one of:"))
        self.assertIn('v1', res.json_body['faultstring'])
        self.assertIn('v2', res.json_body['faultstring'])
        self.assertIn('None', res.json_body['faultstring'])
        self.assertEqual(res.status_int, 400)

    def test_scan_api(self):
        class NS(object):
            @expose(int)
            @validate(int, int)
            def multiply(self, a, b):
                return a * b

        class MyRoot(WSRoot):
            ns = NS()

        r = MyRoot()

        api = list(scan_api(r))
        assert len(api) == 1
        path, fd, args = api[0]
        assert path == ['ns', 'multiply']
        assert fd._wsme_definition.name == 'multiply'
        assert args == []

    def test_scan_subclass(self):
        class MyRoot(WSRoot):
            class SubClass(object):
                pass

        r = MyRoot()
        api = list(scan_api(r))

        assert len(api) == 0

    def test_scan_api_too_deep(self):
        class Loop(object):
            pass

        l = Loop()
        for i in range(0, 21):
            nl = Loop()
            nl.l = l
            l = nl

        class MyRoot(WSRoot):
            loop = l

        r = MyRoot()

        try:
            list(scan_api(r))
            assert False, "ValueError not raised"
        except ValueError as e:
            assert str(e).startswith("Path is too long")

    def test_handle_request(self):
        class MyRoot(WSRoot):
            @expose()
            def touch(self):
                pass

        p = DummyProtocol()
        r = MyRoot(protocols=[p])

        app = webtest.TestApp(r.wsgiapp())

        res = app.get('/')

        assert p.lastreq.path == '/'
        assert p.hits == 1

        res = app.get('/touch?wsmeproto=dummy')

        assert p.lastreq.path == '/touch'
        assert p.hits == 2

        class NoPathProto(DummyProtocol):
            def extract_path(self, request):
                return None

        p = NoPathProto()
        r = MyRoot(protocols=[p])

        app = webtest.TestApp(r.wsgiapp())

        res = app.get('/', expect_errors=True)
        print(res.status, res.body)
        assert res.status_int == 400

    def test_no_available_protocol(self):
        r = WSRoot()

        app = webtest.TestApp(r.wsgiapp())

        res = app.get('/', expect_errors=True)
        assert res.status_int == 500
        print(res.body)
        assert res.body.find(
            b("None of the following protocols can handle this request")) != -1

    def test_return_content_type_guess(self):
        class DummierProto(DummyProtocol):
            content_types = ['text/xml', 'text/plain']

        r = WSRoot([DummierProto()])

        app = webtest.TestApp(r.wsgiapp())

        res = app.get('/', expect_errors=True, headers={
            'Accept': 'text/xml,q=0.8'})
        assert res.status_int == 400
        assert res.content_type == 'text/xml', res.content_type

        res = app.get('/', expect_errors=True, headers={
            'Accept': 'text/plain'})
        assert res.status_int == 400
        assert res.content_type == 'text/plain', res.content_type

    def test_double_expose(self):
        try:
            class MyRoot(WSRoot):
                @expose()
                @expose()
                def atest(self):
                    pass
            assert False, "A ValueError should have been raised"
        except ValueError:
            pass

    def test_multiple_expose(self):
        class MyRoot(WSRoot):
            def multiply(self, a, b):
                return a * b

            mul_int = expose(int, int, int, wrap=True)(multiply)

            mul_float = expose(
                float, float, float,
                wrap=True)(multiply)

            mul_string = expose(
                wsme.types.text, wsme.types.text, int,
                wrap=True)(multiply)

        r = MyRoot(['restjson'])

        app = webtest.TestApp(r.wsgiapp())

        res = app.get('/mul_int?a=2&b=5', headers={
            'Accept': 'application/json'
        })

        self.assertEquals(res.body, b('10'))

        res = app.get('/mul_float?a=1.2&b=2.9', headers={
            'Accept': 'application/json'
        })

        self.assertEquals(res.body, b('3.48'))

        res = app.get('/mul_string?a=hello&b=2', headers={
            'Accept': 'application/json'
        })

        self.assertEquals(res.body, b('"hellohello"'))

    def test_wsattr_mandatory(self):
        class ComplexType(object):
            attr = wsme.types.wsattr(int, mandatory=True)

        class MyRoot(WSRoot):
            @expose(int, body=ComplexType)
            @validate(ComplexType)
            def clx(self, a):
                return a.attr

        r = MyRoot(['restjson'])
        app = webtest.TestApp(r.wsgiapp())
        res = app.post_json('/clx', params={}, expect_errors=True,
                            headers={'Accept': 'application/json'})
        self.assertEqual(res.status_int, 400)

    def test_wsattr_readonly(self):
        class ComplexType(object):
            attr = wsme.types.wsattr(int, readonly=True)

        class MyRoot(WSRoot):
            @expose(int, body=ComplexType)
            @validate(ComplexType)
            def clx(self, a):
                return a.attr

        r = MyRoot(['restjson'])
        app = webtest.TestApp(r.wsgiapp())
        res = app.post_json('/clx', params={'attr': 1005}, expect_errors=True,
                            headers={'Accept': 'application/json'})
        self.assertIn('Cannot set read only field.',
                      res.json_body['faultstring'])
        self.assertIn('1005', res.json_body['faultstring'])
        self.assertEqual(res.status_int, 400)

    def test_wsattr_default(self):
        class ComplexType(object):
            attr = wsme.types.wsattr(wsme.types.Enum(str, 'or', 'and'),
                                     default='and')

        class MyRoot(WSRoot):
            @expose(int)
            @validate(ComplexType)
            def clx(self, a):
                return a.attr

        r = MyRoot(['restjson'])
        app = webtest.TestApp(r.wsgiapp())
        res = app.post_json('/clx', params={}, expect_errors=True,
                            headers={'Accept': 'application/json'})
        self.assertEqual(res.status_int, 400)

    def test_wsproperty_mandatory(self):
        class ComplexType(object):
            def foo(self):
                pass

            attr = wsme.types.wsproperty(int, foo, foo, mandatory=True)

        class MyRoot(WSRoot):
            @expose(int, body=ComplexType)
            @validate(ComplexType)
            def clx(self, a):
                return a.attr

        r = MyRoot(['restjson'])
        app = webtest.TestApp(r.wsgiapp())
        res = app.post_json('/clx', params={}, expect_errors=True,
                            headers={'Accept': 'application/json'})
        self.assertEqual(res.status_int, 400)

    def test_validate_enum_mandatory(self):
        class Version(object):
            number = wsme.types.wsattr(wsme.types.Enum(str, 'v1', 'v2'),
                                       mandatory=True)

        class MyWS(WSRoot):
            @expose(str)
            @validate(Version)
            def setcplx(self, version):
                pass

        r = MyWS(['restjson'])
        app = webtest.TestApp(r.wsgiapp())
        res = app.post_json('/setcplx', params={'version': {}},
                            expect_errors=True,
                            headers={'Accept': 'application/json'})
        self.assertEqual(res.status_int, 400)


class TestFunctionDefinition(unittest.TestCase):

    def test_get_arg(self):
        def myfunc(self):
            pass

        fd = FunctionDefinition(FunctionDefinition)
        fd.arguments.append(FunctionArgument('a', int, True, None))

        assert fd.get_arg('a').datatype is int
        assert fd.get_arg('b') is None



import unittest
from flask import Flask, json, abort
from wsmeext.flask import signature
from wsme.api import Response
from wsme.types import Base, text


class Model(Base):
    id = int
    name = text


class Criterion(Base):
    op = text
    attr = text
    value = text

test_app = Flask(__name__)


@test_app.route('/multiply')
@signature(int, int, int)
def multiply(a, b):
    return a * b


@test_app.route('/divide_by_zero')
@signature(None)
def divide_by_zero():
    return 1 / 0


@test_app.route('/models')
@signature([Model], [Criterion])
def list_models(q=None):
    if q:
        name = q[0].value
    else:
        name = 'first'
    return [Model(name=name)]


@test_app.route('/models/<name>')
@signature(Model, text)
def get_model(name):
    return Model(name=name)


@test_app.route('/models/<name>/secret')
@signature(Model, text)
def model_secret(name):
    abort(403)


@test_app.route('/models/<name>/custom-error')
@signature(Model, text)
def model_custom_error(name):
    class CustomError(Exception):
        code = 412
    raise CustomError("FOO!")


@test_app.route('/models', methods=['POST'])
@signature(Model, body=Model)
def post_model(body):
    return Model(name=body.name)


@test_app.route('/status_sig')
@signature(int, status_code=201)
def get_status_sig():
    return 1


@test_app.route('/status_response')
@signature(int)
def get_status_response():
    return Response(1, status_code=201)


class FlaskrTestCase(unittest.TestCase):

    def setUp(self):
        test_app.config['TESTING'] = True
        self.app = test_app.test_client()

    def tearDown(self):
        pass

    def test_multiply(self):
        r = self.app.get('/multiply?a=2&b=5')
        assert r.data == '10'

    def test_get_model(self):
        resp = self.app.get('/models/test')
        assert resp.status_code == 200

    def test_list_models(self):
        resp = self.app.get('/models')
        assert resp.status_code == 200

    def test_array_parameter(self):
        resp = self.app.get('/models?q.op=%3D&q.attr=name&q.value=second')
        assert resp.status_code == 200
        print resp.data
        self.assertEquals(
            resp.data, '[{"name": "second"}]'
        )

    def test_post_model(self):
        resp = self.app.post('/models', data={"body.name": "test"})
        assert resp.status_code == 200
        resp = self.app.post(
            '/models',
            data=json.dumps({"name": "test"}),
            content_type="application/json"
        )
        assert resp.status_code == 200

    def test_get_status_sig(self):
        resp = self.app.get('/status_sig')
        assert resp.status_code == 201

    def test_get_status_response(self):
        resp = self.app.get('/status_response')
        assert resp.status_code == 201

    def test_custom_clientside_error(self):
        r = self.app.get(
            '/models/test/secret',
            headers={'Accept': 'application/json'}
        )
        assert r.status_code == 403, r.status_code
        assert json.loads(r.data)['faultstring'] == '403: Forbidden'

        r = self.app.get(
            '/models/test/secret',
            headers={'Accept': 'application/xml'}
        )
        assert r.status_code == 403, r.status_code
        assert r.data == ('<error><faultcode>Client</faultcode>'
                          '<faultstring>403: Forbidden</faultstring>'
                          '<debuginfo /></error>')

    def test_custom_non_http_clientside_error(self):
        r = self.app.get(
            '/models/test/custom-error',
            headers={'Accept': 'application/json'}
        )
        assert r.status_code == 412, r.status_code
        assert json.loads(r.data)['faultstring'] == 'FOO!'

        r = self.app.get(
            '/models/test/custom-error',
            headers={'Accept': 'application/xml'}
        )
        assert r.status_code == 412, r.status_code
        assert r.data == ('<error><faultcode>Client</faultcode>'
                          '<faultstring>FOO!</faultstring>'
                          '<debuginfo /></error>')

    def test_serversideerror(self):
        r = self.app.get('/divide_by_zero')
        assert r.status_code == 500
        self.assertEquals(
            r.data,
            '{"debuginfo": null, "faultcode": "Server", "faultstring": '
            '"integer division or modulo by zero"}'
        )

if __name__ == '__main__':
    test_app.run()



from __future__ import absolute_import

import functools
import logging
import sys

import wsme
import wsme.api
import wsme.rest.json
import wsme.rest.xml
import wsme.rest.args
from wsme.utils import is_valid_code

import flask

log = logging.getLogger(__name__)


TYPES = {
    'application/json': wsme.rest.json,
    'application/xml': wsme.rest.xml,
    'text/xml': wsme.rest.xml
}


def get_dataformat():
    if 'Accept' in flask.request.headers:
        for t in TYPES:
            if t in flask.request.headers['Accept']:
                return TYPES[t]

    # Look for the wanted data format in the request.
    req_dataformat = getattr(flask.request, 'response_type', None)
    if req_dataformat in TYPES:
        return TYPES[req_dataformat]

    log.info('''Could not determine what format is wanted by the
             caller, falling back to json''')
    return wsme.rest.json


def signature(*args, **kw):
    sig = wsme.signature(*args, **kw)

    def decorator(f):
        sig(f)
        funcdef = wsme.api.FunctionDefinition.get(f)
        funcdef.resolve_types(wsme.types.registry)

        @functools.wraps(f)
        def wrapper(*args, **kwargs):
            args, kwargs = wsme.rest.args.get_args(
                funcdef, args, kwargs,
                flask.request.args, flask.request.form,
                flask.request.data,
                flask.request.mimetype
            )

            if funcdef.pass_request:
                kwargs[funcdef.pass_request] = flask.request

            dataformat = get_dataformat()

            try:
                result = f(*args, **kwargs)

                # NOTE: Support setting of status_code with default 20
                status_code = funcdef.status_code
                if isinstance(result, wsme.api.Response):
                    status_code = result.status_code
                    result = result.obj

                res = flask.make_response(
                    dataformat.encode_result(
                        result,
                        funcdef.return_type
                    )
                )
                res.mimetype = dataformat.content_type
                res.status_code = status_code
            except:
                try:
                    exception_info = sys.exc_info()
                    orig_exception = exception_info[1]
                    orig_code = getattr(orig_exception, 'code', None)
                    data = wsme.api.format_exception(exception_info)
                finally:
                    del exception_info

                res = flask.make_response(dataformat.encode_error(None, data))
                if data['faultcode'] == 'client':
                    res.status_code = 400
                elif orig_code and is_valid_code(orig_code):
                    res.status_code = orig_code
                else:
                    res.status_code = 500
            return res

        wrapper.wsme_func = f
        return wrapper
    return decorator
